package com.qcg.comp.sdk;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.wsaddressing.W3CEndpointReference;

import com.qcg.comp.sdk.ardl.ARDL;
import com.qcg.comp.sdk.exceptions.InvalidRequestMessageException;
import com.qcg.comp.sdk.exceptions.NoMatchingResourcesException;
import com.qcg.comp.sdk.exceptions.NoSlotsAvailableException;
import com.qcg.comp.sdk.exceptions.NotAuthorizedException;
import com.qcg.comp.sdk.exceptions.UnknownReservationIdentifierException;
import com.qcg.comp.sdk.exceptions.UnsupportedFeatureException;
import com.qcg.comp.sdk.types.ReservationDocument;
import com.qcg.comp.sdk.types.ReservationEndpointReference;
import com.qcg.comp.stubs.ares.ARESReservationDocumentType;
import com.qcg.comp.stubs.ares.ARESReservationStatusType;
import com.qcg.comp.stubs.ares.CreateReservationType;
import com.qcg.comp.stubs.ares.GetActiveReservationsType;
import com.qcg.comp.stubs.ares.GetReservationDocumentResponseType;
import com.qcg.comp.stubs.ares.GetReservationDocumentType;
import com.qcg.comp.stubs.ares.GetReservationStatusType;
import com.qcg.comp.stubs.ares.InvalidRequestMessageFault;
import com.qcg.comp.stubs.ares.NoMatchingResourcesFault;
import com.qcg.comp.stubs.ares.NoSlotsAvailableFault;
import com.qcg.comp.stubs.ares.NotAuthorizedFault;
import com.qcg.comp.stubs.ares.QCGComputingAdvanceReservation;
import com.qcg.comp.stubs.ares.QCGComputingAdvanceReservationPortType;
import com.qcg.comp.stubs.ares.TerminateReservationType;
import com.qcg.comp.stubs.ares.UnknownReservationIdentifierFault;
import com.qcg.comp.stubs.ares.UnsupportedFeatureFault;
import com.qcg.core.sdk.QCGCoreService;
import com.qcg.core.sdk.security.AuthenticationModule;


public class ARES extends QCGCoreService {

	public ARES(String hostname, int port, AuthenticationModule transportLayer,
			AuthenticationModule messageLayer, boolean useWSA)
			throws JAXBException {
		super(hostname, port, transportLayer, messageLayer, useWSA);
	}

	public ARES(String hostname, int port, AuthenticationModule transportLayer,
			AuthenticationModule messageLayer) throws JAXBException {
		super(hostname, port, transportLayer, messageLayer);
	}

	public ARES(String hostname, int port, AuthenticationModule am)
			throws JAXBException {
		super(hostname, port, am);
	}

	public static final String QCG_ARES_NS = "http://schemas.qoscosgrid.org/comp/2011/04/ares";
	protected QCGComputingAdvanceReservationPortType ares;
	
	public ARES(String hostname, int port, AuthenticationModule am,
			boolean useWSA) throws JAXBException{
		super(hostname, port, am, useWSA);
		ares = (QCGComputingAdvanceReservationPortType)stubs;
	}

	@Override
	@SuppressWarnings("all")
	protected List<Handler> getHandlersChain() {
		return null;
	}

	@Override
	protected Object getPort(Service service, WebServiceFeature... features) {
		return ((QCGComputingAdvanceReservation)service).getQCGComputingAdvanceReservation(features);
	}

	@Override
	protected Service getService() {
		return new QCGComputingAdvanceReservation(ARES.class.getResource("/schemas/qcg-comp-ares.wsdl"), new QName(QCG_ARES_NS, "QCGComputingAdvanceReservation"));
	}
	
	public ReservationEndpointReference createReservation(ARDL reservationDocument) throws InvalidRequestMessageException, NoMatchingResourcesException, NoSlotsAvailableException, NotAuthorizedException, UnsupportedFeatureException {
		CreateReservationType parameters = new CreateReservationType();
		
		parameters.setReservationDocument( new ARESReservationDocumentType() );
		parameters.getReservationDocument().setReservationDefinition(reservationDocument.getReservationDefinition());
		
		try {
			return new ReservationEndpointReference(ares.createReservation(parameters).getReservationIdentifier());
		} catch (InvalidRequestMessageFault e) {
			throw new InvalidRequestMessageException(e);
		} catch (NoMatchingResourcesFault e) {
			throw new NoMatchingResourcesException(e);
		} catch (NoSlotsAvailableFault e) {
			throw new NoSlotsAvailableException(e);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnsupportedFeatureFault e) {
			throw new UnsupportedFeatureException(e);
		}
	}
	
	public void terminateReservation(ReservationEndpointReference rEpr) throws NotAuthorizedException, UnknownReservationIdentifierException {
		TerminateReservationType parameters = new TerminateReservationType();
		
		parameters.setReservationIdentifier(rEpr.getEndpointReference(getURL()));
		
		try {
			ares.terminateReservation(parameters);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownReservationIdentifierFault e) {
			throw new UnknownReservationIdentifierException(e);
		}
	}
	
	public ARESReservationStatusType getReservationStatus(ReservationEndpointReference rEpr) throws NotAuthorizedException, UnknownReservationIdentifierException {
		GetReservationStatusType parameters = new GetReservationStatusType();
		
		parameters.setReservationIdentifier(rEpr.getEndpointReference(getURL()));
		
		try {
			return ares.getReservationStatus(parameters).getReservationStatus();
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownReservationIdentifierFault e) {
			throw new UnknownReservationIdentifierException(e);
		}
	}
	
	public ReservationDocument getReservationDocument(ReservationEndpointReference rEpr) throws NotAuthorizedException, UnknownReservationIdentifierException {
		GetReservationDocumentType parameters = new GetReservationDocumentType();
		
		parameters.setReservationIdentifier(rEpr.getEndpointReference(getURL()));
		GetReservationDocumentResponseType rdt;
		try {
			rdt = ares.getReservationDocument(parameters);
			return new ReservationDocument(new ARDL(rdt.getReservationDocument().getReservationDefinition()), rdt.getLocalReservationId());
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownReservationIdentifierFault e) {
			throw new UnknownReservationIdentifierException(e);
		}
	}
	
	public List<ReservationEndpointReference> getActiveReservations() throws NotAuthorizedException {
		
		List<W3CEndpointReference> w3cList;
		try {
			w3cList = ares.getActiveReservations(new GetActiveReservationsType()).getReservationIdentifier();
			List<ReservationEndpointReference> fedList = new ArrayList<ReservationEndpointReference>(w3cList.size());
			
			for (W3CEndpointReference epr : w3cList) {
				fedList.add(new ReservationEndpointReference(epr));
			}
			
			return fedList;
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		}

	}

}
