package com.qcg.comp.sdk;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.MessageContext;

import org.metastatic.rsync.Checksum32;
import org.metastatic.rsync.ChecksumPair;
import org.metastatic.rsync.Configuration;
import org.metastatic.rsync.GeneratorEvent;
import org.metastatic.rsync.GeneratorListener;
import org.metastatic.rsync.GeneratorStream;
import org.metastatic.rsync.JarsyncProvider;
import org.metastatic.rsync.ListenerException;
import org.metastatic.rsync.Rdiff;

import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.rsync.DeltaDataSource;
import com.qcg.comp.sdk.rsync.DeltaInputStream;
import com.qcg.comp.sdk.rsync.GetDeltaHandler;
import com.qcg.comp.sdk.rsync.Params;
import com.qcg.comp.stubs.rsync.BlockSignatureType;
import com.qcg.comp.stubs.rsync.CommonType;
import com.qcg.comp.stubs.rsync.FileType;
import com.qcg.comp.stubs.rsync.FilesType;
import com.qcg.comp.stubs.rsync.IOFault;
import com.qcg.comp.stubs.rsync.InternalFault;
import com.qcg.comp.stubs.rsync.InvalidPathFault;
import com.qcg.comp.stubs.rsync.RsyncFault;
import com.qcg.comp.stubs.rsync.QCGComputingRsync;
import com.qcg.comp.stubs.rsync.QCGComputingRsyncPortType;
import com.qcg.comp.stubs.rsync.SignatureType;
import com.qcg.comp.stubs.staging.FileNotFoundFault;
import com.qcg.comp.stubs.staging.NotAuthorizedFault;
import com.qcg.comp.stubs.staging.StagingFault;
import com.qcg.core.sdk.QCGCoreService;
import com.qcg.core.sdk.attachments.AttachmentHandler;
import com.qcg.core.sdk.security.AuthenticationModule;

public class QCGRsync extends QCGCoreService {

	public static final String QCG_RSYNC_NS = "http://schemas.qoscosgrid.org/comp/2011/04/rsync";

	protected static final sun.misc.BASE64Encoder BASE64_ENCODER = new sun.misc.BASE64Encoder();
	protected static final sun.misc.BASE64Decoder BASE64_DECODER = new sun.misc.BASE64Decoder();

	protected QCGComputingRsyncPortType rsync;
	protected QCGStaging staging;

	protected AttachmentHandler attachmentHandler;

	protected Rdiff rdiff = new Rdiff();
	protected static DatatypeFactory dtFactory = null; 

	
	static {
		Security.addProvider(new JarsyncProvider());
		try {
			dtFactory = DatatypeFactory.newInstance();
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}
	}
	
	public QCGRsync(String hostname, int port,
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer, boolean useWSA)
			throws JAXBException {
		super(hostname, port, transportLayer, messageLayer, useWSA);
		this.rsync = (QCGComputingRsyncPortType) stubs;
		
		staging = new QCGStaging(hostname, port, transportLayer, messageLayer, useWSA, null);

	}

	public QCGRsync(String hostname, int port,
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer) throws JAXBException {
		super(hostname, port, transportLayer, messageLayer);
		this.rsync = (QCGComputingRsyncPortType) stubs;
		
		staging = new QCGStaging(hostname, port, transportLayer, messageLayer, null);
	}

	public QCGRsync(String hostname, int port, AuthenticationModule am)
			throws JAXBException {
		super(hostname, port, am);
		this.rsync = (QCGComputingRsyncPortType) stubs;
		
		staging = new QCGStaging(hostname, port, am, null);
	}

	public QCGRsync(String hostname, int port, AuthenticationModule am,
			boolean useWSA) throws JAXBException {
		super(hostname, port, am, useWSA);
		this.rsync = (QCGComputingRsyncPortType) stubs;
		
		staging = new QCGStaging(hostname, port, am, null);
	}
	
	private class LazyInputStream extends InputStream {

		private FileInputStream file = null;

		public LazyInputStream(String filename) {
			try {
				file = new FileInputStream(filename);
			} catch (FileNotFoundException e) {
			}
		}

		@Override
		public int read() throws IOException {
			return file != null ? file.read() : -1;
		}

		@Override
		public int read(byte[] b) throws IOException {
			return file != null ? file.read(b) : -1;
		}

		@Override
		public int read(byte[] b, int off, int len) throws IOException {
			return file != null ? file.read(b, off, len) : -1;
		}


		@Override
		public void close() throws IOException {
			if (file != null) 
				file.close();
		}

	}


	@Override
	protected Object getPort(Service service, WebServiceFeature... features) {
		return ((QCGComputingRsync) service).getQCGComputingRsync(features);
	}

	@Override
	protected Service getService() {
		return new QCGComputingRsync(
				QCGRsync.class.getResource("/schemas/qcg-comp-rsync.wsdl"),
				new QName(QCG_RSYNC_NS, "QCGComputingRsync"));
	}

	private IOException stagingException;

	@Override
	@SuppressWarnings("all")
	protected List<Handler> getHandlersChain() {
		attachmentHandler = new AttachmentHandler(
				new AttachmentHandler.IOExceptionListener() {

					public void exceptionThrown(IOException ex) {
						stagingException = ex;
					}
				});

		List<Handler> handlers = new ArrayList<Handler>();
		handlers.add(attachmentHandler);

		return handlers;
	}

	protected void checkForIOException() throws IOException {
		if (stagingException != null) {
			IOException ioEx = stagingException;
			stagingException = null;
			throw ioEx;
		}
	}

	protected String joinPath(String dir, String file) {
		// The File.pathSeparator is not used to ensure compatibility with
		// qcg-comp-rsync, which has / hardcoded
		if (!dir.endsWith("/"))
			dir += "/";
		return dir + file;
	}

	private FilesType getLocalFiles(String basedir, String path) 
			throws DatatypeConfigurationException, IOException {
		FilesType localFiles = new FilesType();
		File dir = new File(joinPath(basedir, path));
		
		for (String s : dir.list()) {
			s = joinPath(path, s);
			File f = new File(joinPath(basedir, s));
			
			if (f.isFile() && !f.canRead()) {
				throw new IOException("Unable to read file: " + f.getAbsolutePath());
			}
			
			if (f.isDirectory()) {
				List<FileType> subFiles = getLocalFiles(basedir, s).getFile();
				
				if (subFiles.size() == 0) {
					FileType emptyDir = new FileType();
				
					emptyDir.setPath(s);
					emptyDir.setMTime(timestampToXML(f.lastModified()));
					emptyDir.setIsDirectory(true);
					emptyDir.setIsExecutable(false);
					
					localFiles.getFile().add(emptyDir);
				}
				
				localFiles.getFile().addAll(subFiles);
			}  else {
				FileType file = new FileType();
				
				file.setPath(s);
				file.setMTime(timestampToXML(f.lastModified()));
				file.setIsDirectory(false);
				file.setIsExecutable(f.canExecute());
				
				localFiles.getFile().add(file);
			}
		
		}
		
		return localFiles;
	}

	private XMLGregorianCalendar timestampToXML(long time)
			throws DatatypeConfigurationException {

		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(new Date(time));
		XMLGregorianCalendar xml = dtFactory.newXMLGregorianCalendar(cal);
		
		return xml;
	}

	private class SignatureGeneratorListener implements GeneratorListener {

		private SignatureType result;

		public SignatureGeneratorListener(SignatureType result) {
			this.result = result;
		}

		public void update(GeneratorEvent event) throws ListenerException {
			ChecksumPair pair = event.getChecksumPair();
			BlockSignatureType blockSig = new BlockSignatureType();
			
			blockSig.setWeak(pair.getWeak());
			blockSig.setStrong(BASE64_ENCODER.encode(pair.getStrong()));
			
			result.getSignatures().add(blockSig);
		}

	}

	private SignatureType GetSignature(String fullpath) throws IOException,
			NoSuchAlgorithmException {
		Configuration c = new Configuration();
		
		c.strongSum = MessageDigest.getInstance("MD4");
		c.weakSum = new Checksum32(Params.CHAR_OFFSET);
		c.blockLength = Params.BLOCK_LENGTH;
		c.strongSumLength = Params.STRONG_SUM_LENGTH;
		
		SignatureType result = new SignatureType();
		result.setBlockLength(c.blockLength);
		
		File f = new File(fullpath);
		
		result.setLength((int) f.length());
		result.setRemainder((int) f.length() % c.blockLength);
		result.setStrongSumLength(c.strongSumLength);
		
		GeneratorStream gen = new GeneratorStream(c);
		gen.addListener(new SignatureGeneratorListener(result));
		
		int len = 0;
		byte[] buf = new byte[Params.CHUNK_SIZE];

		InputStream in = new LazyInputStream(fullpath);
		
		while ((len = in.read(buf)) != -1) {
			try {
				gen.update(buf, 0, len);
			} catch (ListenerException le) {
				throw (IOException) le.getCause();
			}
		}
		
		try {
			gen.doFinal();
		} catch (ListenerException le) {
			throw (IOException) le.getCause();
		}
		
		return result;
	}

	private void putDelta(CommonType common, String localBasedir, String path)
			throws IOFault, InternalFault, RsyncFault, IOException,
			NoSuchAlgorithmException {
		
		System.out.println("Sending file: " + path);
		
		List<ChecksumPair> signatures = getSignatures(common, path);
		
		Map<String, DataHandler> attachments = new HashMap<String, DataHandler>();
		DataSource delta = new DeltaDataSource(new DeltaInputStream(signatures, new LazyInputStream(joinPath(localBasedir, path))));
		
		attachments.put("delta", new DataHandler(delta));
		
		((BindingProvider) rsync).getRequestContext().put(MessageContext.OUTBOUND_MESSAGE_ATTACHMENTS, attachments);
		rsync.putDelta(common, path);
		((BindingProvider) rsync).getRequestContext().put(MessageContext.OUTBOUND_MESSAGE_ATTACHMENTS, null);
		
		checkForIOException();
	}

	private List<ChecksumPair> getSignatures(CommonType common, String path)
			throws IOFault, InternalFault, RsyncFault, IOException {
		
		List<ChecksumPair> signatures = new ArrayList<ChecksumPair>();
		long offset = 0;
		
		SignatureType sig = rsync.getSignature(common, path);
		
		for (BlockSignatureType block : sig.getSignatures()) {
			signatures.add(new ChecksumPair(block.getWeak(), BASE64_DECODER.decodeBuffer(block.getStrong()), offset));
			offset += sig.getBlockLength();
		}
		
		return signatures;
	}

	private void getDelta(CommonType common, String localBasedir, String path)
			throws NoSuchAlgorithmException, IOException {
		
		System.out.println("Receiving file: " + path);
		
		String fullpath = new File(joinPath(localBasedir, path)).getAbsolutePath();
		SignatureType sig = GetSignature(fullpath);
		
		attachmentHandler.setFileStagingHandler(new GetDeltaHandler(fullpath));
		rsync.getDelta(common, path, sig);
		attachmentHandler.setFileStagingHandler(null);
		
		checkForIOException();		
	}

	private boolean fileOk(String path, List<Pattern> exclude) {

		if (exclude != null) {


			String pathTokens[] = path.split("/");
			
			for (int i=0; i < pathTokens.length; i++)
				for (Pattern p : exclude) {
	
					if (p.matcher(pathTokens[i]).matches()) {
						System.out.println("Ignoring file " + path + " because of " + p);
						return false;
					}
	
				}
		}

		return true;
	}

	/**
	 * two-way r-synchronize 
	 **/
	public void synchronize(JSDL jsdl, String localDirectory, List<Pattern> exclude) 
					throws 
					InternalFault, 
					InvalidPathFault,
					DatatypeConfigurationException, 
					NoSuchAlgorithmException,
					IOException, 
					IOFault, 
					RsyncFault, 
					FileNotFoundFault, 
					NotAuthorizedFault, 
					StagingFault {
		
		CommonType params = new CommonType();
		
		
		params.setBaseDir(jsdl.getWorkingDirectory());
		params.setUserName(jsdl.getUserName());
		
		FilesType remoteFiles = rsync.getFiles(params);
		FilesType localFiles = getLocalFiles(localDirectory, "");
		
		for (FileType local : localFiles.getFile()) {
			boolean found = false;

			if (!fileOk(local.getPath(), exclude))
				continue;

			for (FileType remote : remoteFiles.getFile()) {
				if (local.getPath().equals(remote.getPath())) {
					found = true;
					if (local.getMTime().compare(remote.getMTime()) == DatatypeConstants.LESSER)
						getDelta(params, localDirectory, local.getPath());
					else
						putDelta(params, localDirectory, local.getPath());
				}
			}
			if (!found) {
				if (! local.isIsDirectory()) {
					putDelta(params, localDirectory, local.getPath());
					
					if (local.isIsExecutable())
						staging.chmod(jsdl.getWorkingDirectory() + "/" + local.getPath(), "0755", jsdl.getUserName());
					
				} else {
					staging.mkdir(jsdl.getWorkingDirectory() + local.getPath(), "0755", jsdl.getUserName());
				}
			}
		}
		
		for (FileType remote : remoteFiles.getFile()) {
			boolean found = false;

			if (!fileOk(remote.getPath(), exclude))
				continue;

			for (FileType local : localFiles.getFile()) {
				if (local.getPath().equals(remote.getPath())) {
					found = true;
					break;
				}
			}
			
			if (!found)
				getDelta(params, localDirectory, remote.getPath());
		}
	}

	protected List<Pattern> globToRegex(List<String> input) {
		List<Pattern> result = new ArrayList<Pattern>();

		for (String s : input) {
			s = s.replace(".", "\\.");
			s = s.replace("?", ".");
			s = s.replace("*", ".*");
			s = "^(.*/|)" + s + "(/.*|)$";
			result.add(Pattern.compile(s));
		}

		return result;
	}

	/***
	 * cmdline rsync compatible version of localToRemote
	 * @param jsdl
	 * @param localDirectory
	 * @param exclude
	 * @throws InternalFault
	 * @throws InvalidPathFault
	 * @throws DatatypeConfigurationException
	 * @throws NoSuchAlgorithmException
	 * @throws IOException
	 * @throws IOFault
	 * @throws RsyncFault
	 * @throws StagingFault 
	 * @throws NotAuthorizedFault 
	 * @throws FileNotFoundFault 
	 */
	public void rsyncLocalToRemote(JSDL jsdl, String localDirectory,
			List<String> exclude) throws InternalFault, InvalidPathFault,
			DatatypeConfigurationException, NoSuchAlgorithmException,
			IOException, IOFault, RsyncFault, FileNotFoundFault, NotAuthorizedFault, StagingFault {

		List<Pattern> regexExclude = globToRegex(exclude);

		Security.addProvider(new JarsyncProvider());
		CommonType params = new CommonType();
		params.setBaseDir(jsdl.getWorkingDirectory());
		params.setUserName(jsdl.getUserName());
		if (new File(localDirectory).isDirectory()) {
			params.setBaseDir(jsdl.getWorkingDirectory() + "/" + localDirectory);
			FilesType localFiles = getLocalFiles(localDirectory, "");
			for (FileType local : localFiles.getFile()) {

				if (!fileOk(local.getPath(), regexExclude))
					continue;
				
				if (! local.isIsDirectory()) {
					putDelta(params, localDirectory, local.getPath());
				} else {
					staging.mkdir(jsdl.getWorkingDirectory()  + "/" + localDirectory + local.getPath(), "0755", jsdl.getUserName());
				}

			}
		} else if (fileOk(localDirectory, regexExclude))
			putDelta(params, "./", localDirectory);
	}

	/***
	 * one-way (update remote site) synchronize
	 * @param jsdl
	 * @param localDirectory
	 * @param exclude
	 * @throws InternalFault
	 * @throws InvalidPathFault
	 * @throws DatatypeConfigurationException
	 * @throws NoSuchAlgorithmException
	 * @throws IOException
	 * @throws IOFault
	 * @throws RsyncFault
	 * @throws FileNotFoundFault
	 * @throws NotAuthorizedFault
	 * @throws StagingFault
	 */
	public void localToRemote(JSDL jsdl, String localDirectory, List<Pattern> exclude) 
					throws 
					InternalFault, 
					InvalidPathFault,
					DatatypeConfigurationException, 
					NoSuchAlgorithmException,
					IOException, 
					IOFault, 
					RsyncFault, 
					FileNotFoundFault, 
					NotAuthorizedFault, 
					StagingFault {
		Set<String> createdDirectories = new HashSet<String>(); 
		CommonType params = new CommonType();

		params.setBaseDir(jsdl.getWorkingDirectory());
		params.setUserName(jsdl.getUserName());
		
		FilesType localFiles = getLocalFiles(localDirectory, "");
		
		for (FileType local : localFiles.getFile()) {

			if (!fileOk(local.getPath(), exclude))
				continue; 
			
			if (! local.isIsDirectory()) {
				putDelta(params, localDirectory, local.getPath());
			} else if (!createdDirectories.contains(local.getPath())) {
				String subDir = "";
				String[] dirTokens = local.getPath().split("/");
				
				System.err.println("mkdir: " + jsdl.getWorkingDirectory() + local.getPath());
				staging.mkdir(jsdl.getWorkingDirectory() + local.getPath(), "0755", jsdl.getUserName());
				
				for (String dirToken: dirTokens) {
					subDir += "/" + dirToken;
					createdDirectories.add(subDir);
					System.err.println("add Dir:" + subDir);
				}
			}
		}

	}

	/***
	 * one-way (update local site) synchronize
	 * @param jsdl
	 * @param localDirectory
	 * @param exclude
	 * @throws InternalFault
	 * @throws InvalidPathFault
	 * @throws DatatypeConfigurationException
	 * @throws NoSuchAlgorithmException
	 * @throws IOException
	 * @throws IOFault
	 * @throws RsyncFault
	 */
	public void remoteToLocal(JSDL jsdl, String localDirectory, List<Pattern> exclude) 
					throws 
					InternalFault, 
					InvalidPathFault,
					DatatypeConfigurationException, 
					NoSuchAlgorithmException,
					IOException, 
					IOFault, 
					RsyncFault {
		
		CommonType params = new CommonType();
		
		params.setBaseDir(jsdl.getWorkingDirectory());
		params.setUserName(jsdl.getUserName());
		
		FilesType remoteFiles = rsync.getFiles(params);

		for (FileType remote : remoteFiles.getFile()) {

			if (fileOk(remote.getPath(), exclude))
				getDelta(params, localDirectory, remote.getPath());

		}
	}
}
