package com.qcg.comp.sdk.jsdl;

import java.io.Reader;
import java.io.Serializable;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import com.qcg.comp.sdk.types.ReservationEndpointReference;
import com.qcg.comp.stubs.jsdl.ApplicationType;
import com.qcg.comp.stubs.jsdl.BoundaryType;
import com.qcg.comp.stubs.jsdl.CPUArchitectureType;
import com.qcg.comp.stubs.jsdl.CandidateHostsType;
import com.qcg.comp.stubs.jsdl.DataStagingType;
import com.qcg.comp.stubs.jsdl.ExactType;
import com.qcg.comp.stubs.jsdl.JobDefinitionType;
import com.qcg.comp.stubs.jsdl.JobDescriptionType;
import com.qcg.comp.stubs.jsdl.JobIdentificationType;
import com.qcg.comp.stubs.jsdl.ObjectFactory;
import com.qcg.comp.stubs.jsdl.OperatingSystemType;
import com.qcg.comp.stubs.jsdl.OperatingSystemTypeEnumeration;
import com.qcg.comp.stubs.jsdl.OperatingSystemTypeType;
import com.qcg.comp.stubs.jsdl.ProcessorArchitectureEnumeration;
import com.qcg.comp.stubs.jsdl.RangeValueType;
import com.qcg.comp.stubs.jsdl.ResourcesType;
import com.qcg.comp.stubs.jsdl.ares.ReservationType;
import com.qcg.comp.stubs.jsdl.hpcpa.ArgumentType;
import com.qcg.comp.stubs.jsdl.hpcpa.DirectoryNameType;
import com.qcg.comp.stubs.jsdl.hpcpa.EnvironmentType;
import com.qcg.comp.stubs.jsdl.hpcpa.FileNameType;
import com.qcg.comp.stubs.jsdl.hpcpa.HPCProfileApplicationType;
import com.qcg.comp.stubs.jsdl.hpcpa.UserNameType;


public class JSDL implements Serializable, Cloneable{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 200L;
	
	protected static JAXBContext jaxbCtxt;

	protected String jobName;
	protected String jobDescription;
	
	protected String applicationName;
	protected String applicationVersion;

	protected String executable;
	protected String error;
	protected String input;
	protected String output;
	protected String userName;
	protected String workingDirectory;
	protected String nativeSpecification;
	protected List<String> candidateHosts = new LinkedList<String>();
	protected Integer minCpu;
	protected Integer maxCpu;
	protected String operatingSystem;
	protected String cpuArch;
	protected String queueName;

	protected List<String> arguments = new LinkedList<String>();
	protected Map<String, String> environment = new HashMap<String, String>();
	
	protected ReservationEndpointReference reservationIdentifier;
	
	protected List<JSDLDataStaging> dataStaging = new LinkedList<JSDLDataStaging>();

	protected transient JobDefinitionType job;
	protected transient String strValue;

	public JSDL(JobDefinitionType jobDef) {
		job = jobDef;
		convertFromJobDefinition();
	}
	
	protected static synchronized JAXBContext getJAXBContext() throws JAXBException {
		
		if (jaxbCtxt == null) {
			jaxbCtxt = JAXBContext.newInstance(ObjectFactory.class.getPackage().getName());
		}
		
		return jaxbCtxt;
	}
	
	@SuppressWarnings("unchecked")
	public JSDL(Reader jsdlReader) throws JAXBException {

		Unmarshaller um = getJAXBContext().createUnmarshaller();
		job = ((JAXBElement<JobDefinitionType>)um.unmarshal(jsdlReader)).getValue(); 
		convertFromJobDefinition();
	}
	
	public JSDL(String jobName) {
		this.jobName = jobName;
	}
	
	protected void convertFromJobDefinition() {
		if  (job.getJobDescription().getJobIdentification()!= null) {
			jobName = job.getJobDescription().getJobIdentification().getJobName();
			jobDescription = job.getJobDescription().getJobIdentification().getDescription();
		}
		
		if  (job.getJobDescription().getApplication() != null) {

			applicationName = job.getJobDescription().getApplication().getApplicationName();
			applicationVersion = job.getJobDescription().getApplication().getApplicationVersion();
			
			if  (job.getJobDescription().getApplication().getHPCProfileApplication()!= null) {
				HPCProfileApplicationType app = job.getJobDescription().getApplication().getHPCProfileApplication();
				
				if (app.getExecutable() != null)
					executable = app.getExecutable().getValue();
				
				for (ArgumentType at : app.getArgument()) {
					arguments.add(at.getValue());
				}
				
				input = app.getInput() != null ? app.getInput().getValue() : null;
				output = app.getOutput() != null ? app.getOutput().getValue() : null;
				error = app.getError() != null ? app.getError().getValue() : null;
			 	workingDirectory = app.getWorkingDirectory() != null ? app.getWorkingDirectory().getValue() : null;
			 	userName = app.getUserName() != null ? app.getUserName().getValue() : null;
				
				for (EnvironmentType et  : app.getEnvironment()) {
					environment.put(et.getName(), et.getValue());
				}
			}
		}
		
		if (job.getJobDescription().getResources() != null) {
			ResourcesType res = job.getJobDescription().getResources();
			
			if (res.getReservation() != null) 
				reservationIdentifier = new ReservationEndpointReference(job.getJobDescription().getResources().getReservation().getReservationIdentifier());
			else
				reservationIdentifier = null;
			
			if (res.getNativeSpecification() != null) {
				nativeSpecification = res.getNativeSpecification();
			} else {
				nativeSpecification = null;
			}
			
			if (res.getQueue() != null) {
				queueName = res.getQueue();
			} else {
				queueName = null;
			}
			
			if (res.getTotalCPUCount() != null) {
				if (res.getTotalCPUCount().getExact().size() == 1) {
					minCpu = maxCpu = new Integer((int)res.getTotalCPUCount().getExact().get(0).getValue());
				} 
				
				if (res.getTotalCPUCount().getLowerBoundedRange() != null) {
					minCpu = new Integer((int)res.getTotalCPUCount().getLowerBoundedRange().getValue());
				}
				
				if (res.getTotalCPUCount().getUpperBoundedRange() != null) {
					maxCpu = new Integer((int)res.getTotalCPUCount().getUpperBoundedRange().getValue());
				}
			} else {
				minCpu = null;
				maxCpu = null;
			}
			
			if (res.getCPUArchitecture() != null) {
				cpuArch = res.getCPUArchitecture().getCPUArchitectureName().value();
			} else {
				cpuArch = null;
			}
			
			if (res.getOperatingSystem() != null) {
				operatingSystem = res.getOperatingSystem().getOperatingSystemType().getOperatingSystemName().value();
			} else {
				operatingSystem = null;
			}
			
			if (res.getCandidateHosts() != null) {
				candidateHosts.addAll(res.getCandidateHosts().getHostName());
			}
			
		}
		
		for (DataStagingType ds : job.getJobDescription().getDataStaging()) {
			dataStaging.add(new JSDLDataStaging(ds));
		}
		
	}
	
	protected void convertToJobDefinition() {
		if (job!= null)
			return;
		
		job = new JobDefinitionType();

		JobDescriptionType jobDesc = new JobDescriptionType();
		job.setJobDescription(jobDesc);
		
		if (jobName != null) {
			jobDesc.setJobIdentification(new JobIdentificationType());
			jobDesc.getJobIdentification().setJobName(jobName);
		}
		
		if (jobDescription != null) {
			if (jobDesc.getJobIdentification() == null) {
				jobDesc.setJobIdentification(new JobIdentificationType());
			}
			
			jobDesc.getJobIdentification().setDescription(jobDescription);
		}
		
		if (applicationName != null) {
			jobDesc.setApplication(new ApplicationType());
			jobDesc.getApplication().setApplicationName(applicationName);
			jobDesc.getApplication().setApplicationVersion(applicationVersion);
		}
		
		if (executable != null ||
				error != null ||
				input != null ||
				output!= null ||
				workingDirectory != null ||
				userName != null ||
				arguments.size() != 0 ||
				environment.size() != 0) {
			
			HPCProfileApplicationType app = new HPCProfileApplicationType();
			jobDesc.setApplication( jobDesc.getApplication() == null ? new ApplicationType() : jobDesc.getApplication());
			jobDesc.getApplication().setHPCProfileApplication(app);
			
			app.setExecutable(createFileNameType(executable));
			app.setInput(createFileNameType(input));
			app.setOutput(createFileNameType(output));
			app.setError(createFileNameType(error));
			
			
			if (workingDirectory != null) {
				app.setWorkingDirectory(new DirectoryNameType());
				app.getWorkingDirectory().setValue(workingDirectory);
			}
			
			if (userName != null) {
				app.setUserName(new UserNameType());
				app.getUserName().setValue(userName);
			}
			
			for (String arg : arguments) {
				ArgumentType at = new ArgumentType();
				at.setValue(arg);
				app.getArgument().add(at);
			}
			
			for (String varName : environment.keySet()) {
				String varValue = environment.get(varName);
				EnvironmentType et = new EnvironmentType();
				
				et.setName(varName);
				et.setValue(varValue);
				
				app.getEnvironment().add(et);
			}		
		}
		
		if (reservationIdentifier != null || 
				nativeSpecification != null ||
				candidateHosts.size() != 0 ||
				cpuArch != null ||
				operatingSystem != null ||
				minCpu != null ||
				maxCpu != null ||
				queueName != null) {
			jobDesc.setResources(new ResourcesType());	
		}
		
		if (reservationIdentifier != null) {
			jobDesc.getResources().setReservation(new ReservationType());
			jobDesc.getResources().getReservation().setReservationIdentifier(reservationIdentifier.getEndpointReference());
		}
		
		if (nativeSpecification != null) {
			jobDesc.getResources().setNativeSpecification(nativeSpecification);
		}
		
		if (candidateHosts.size() > 0) {
			jobDesc.getResources().setCandidateHosts(new CandidateHostsType());
			jobDesc.getResources().getCandidateHosts().getHostName().addAll(candidateHosts);
		}
		
		if (cpuArch != null) {
			CPUArchitectureType cat = new CPUArchitectureType();
			cat.setCPUArchitectureName(ProcessorArchitectureEnumeration.valueOf(cpuArch));
			jobDesc.getResources().setCPUArchitecture(cat);
		}
		
		if (operatingSystem != null) {
			OperatingSystemType ost =  new OperatingSystemType();
			ost.setOperatingSystemType(new OperatingSystemTypeType());
			ost.getOperatingSystemType().setOperatingSystemName(OperatingSystemTypeEnumeration.valueOf(operatingSystem));
		}
		
		if (minCpu != null || maxCpu != null) {
			RangeValueType rvt = new RangeValueType();
			
			if (minCpu == null)
				minCpu = maxCpu;
			else if (maxCpu == null)
				maxCpu = minCpu;
			
			if (minCpu.equals(maxCpu)) {
				ExactType et = new ExactType();
				et.setValue(minCpu.doubleValue());
				rvt.getExact().add(et);
			} else {
				BoundaryType lb = new BoundaryType();
				lb.setValue(minCpu.doubleValue());
				
				rvt.setLowerBoundedRange(lb);
				
				BoundaryType ub = new BoundaryType();
				ub.setValue(maxCpu.doubleValue());
				
				rvt.setUpperBoundedRange(ub);
			}
			
			jobDesc.getResources().setTotalCPUCount(rvt);
		}
		
		if (queueName != null) {
			jobDesc.getResources().setQueue(queueName);
		}
		
		for (JSDLDataStaging ds : dataStaging) {
			jobDesc.getDataStaging().add(ds.getDataStagingElement());
		}
		
	}

	protected FileNameType createFileNameType(String fileName) {
		if (fileName == null)
			return null;
		
		FileNameType fn = new FileNameType();
		fn.setValue(fileName);
		
		return fn;
	}
	
	public String getApplicationName() {
		return applicationName;
	}
	public String getApplicationVersion() {
		return applicationVersion;
	}
	
	public List<String> getArguments() {
		invalidate();
		
		return arguments;
	}
	
	public List<JSDLDataStaging> getDataStaging() {
		invalidate();

		return dataStaging;
	}
	
	public Map<String, String> getEnvironment() {
		invalidate();
		
		return environment;
	}
	
	public String getError() {
		
		return error;
	}
	
	public String getExecutable() {
		
		return executable;
	}
	
	public String getInput() {
		return input;
	}
	
	public String getJobDescription() {
		
		return jobDescription;
	}
	
	public String getJobName() {
		
		return jobName;
	}
	
	public String getOutput() {
		
		return output;
	}

	public ReservationEndpointReference getReservationIdentifier() {
		
		return reservationIdentifier;
	}

	public String getUserName() {
		
		return userName;
	}

	public String getWorkingDirectory() {

		return workingDirectory;
	}
	
	public String getNativeSpecification() {
		
		return nativeSpecification;
	}

	protected void invalidate() {
		strValue = null;
		job = null;
	}

	public void setApplicationName(String applicationName) {
		invalidate();

		this.applicationName = applicationName;
	}

	public void setApplicationVersion(String applicationVersion) {
		invalidate();

		this.applicationVersion = applicationVersion;
	}

	public void setError(String error) {
		invalidate();

		this.error = error;
	}

	public void setExecutable(String executable) {
		invalidate();
		
		this.executable = executable;
	}

	public void setInput(String input) {
		invalidate();

		this.input = input;
	}

	public void setJobDescription(String jobDescription) {
		invalidate();

		this.jobDescription = jobDescription;
	}

	public void setJobName(String jobName) {
		invalidate();

		this.jobName = jobName;
	}

	public void setOutput(String output) {
		invalidate();

		this.output = output;
	}

	public void setReservationIdentifier(ReservationEndpointReference reservationIdentifier) {
		invalidate();
		
		this.reservationIdentifier = reservationIdentifier;
	}

	public void setUserName(String userName) {
		invalidate();

		this.userName = userName;
	}
	
	public void setNativeSpecification(String nativeSpecification) {
		invalidate();

		this.nativeSpecification = nativeSpecification;
	}

	public void setWorkingDirectory(String workingDirectory) {
		invalidate();

		this.workingDirectory = workingDirectory;
	}

	public JobDefinitionType toJobDefinition() {
		convertToJobDefinition();
		return job;
	}

	public Integer getMinCpu() {
		return minCpu;
	}

	public void setMinCpu(Integer minCpu) {
		invalidate();
		
		this.minCpu = minCpu;
	}

	public Integer getMaxCpu() {
		return maxCpu;
	}

	public void setMaxCpu(Integer maxCpu) {
		invalidate();
		
		this.maxCpu = maxCpu;
	}

	public String getOperatingSystem() {
		return operatingSystem;
	}

	public void setOperatingSystem(String operatingSystem) {
		invalidate();
		
		this.operatingSystem = operatingSystem;
	}

	public String getCpuArch() {
		return cpuArch;
	}

	public void setCpuArch(String cpuArch) {
		invalidate();
		
		this.cpuArch = cpuArch;
	}
	
	public String getQueueName() {
		return queueName;
	}

	public void setQueueName(String queueName) {
		invalidate();
		
		this.queueName = queueName;
	}

	public List<String> getCandidateHosts() {
		invalidate();
		
		return candidateHosts;
	}

	public String toString() {
		
		try {
			if (strValue == null) {
				convertToJobDefinition();
				Marshaller m = getJAXBContext().createMarshaller();
				m.setProperty("jaxb.formatted.output",new Boolean(true)); 

				StringWriter sw  = new StringWriter(512);
				m.marshal((new ObjectFactory()).createJobDefinition(job), sw);
				strValue = sw.getBuffer().toString();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			strValue = "Unable to marshal JSDL:" + ex.getMessage();
		}
		
		return strValue;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		String jsdlStr = this.toString();
		try {
			return new JSDL(new StringReader(jsdlStr));
		} catch (JAXBException e) {
			e.printStackTrace();
			return null;
		}
	}

}
