package com.qcg.comp.sdk.types;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;

import javax.xml.ws.wsaddressing.W3CEndpointReference;

import com.qcg.core.sdk.utils.EndpointReferenceUtils;

public class ReservationEndpointReference implements Serializable{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 200L;
	
	protected String address;
	protected String reservationUUID;
	protected transient W3CEndpointReference endpointReference;
	
	public String getAddress() {
		return address;
	}

	public String getReservationUUID() {
		return reservationUUID;
	}

	public W3CEndpointReference getEndpointReference() {
		return this.getEndpointReference(null);
	}
	
	public W3CEndpointReference getEndpointReference(String serviceAddress){
		address  = address == null ? serviceAddress : address;

		if (endpointReference == null) {
			
			StringBuffer sb = new StringBuffer(200);
			
			sb.append("<wsa5:EndpointReference xmlns:wsa5='http://www.w3.org/2005/08/addressing'>");
			sb.append("<wsa5:Address>");
			sb.append(address);
			sb.append("</wsa5:Address>");
			sb.append("<wsa5:ReferenceParameters>");
			sb.append("<ns:ReservationId xmlns:ns='http://schemas.qoscosgrid.org/comp/2011/04/ares'>");
			sb.append(reservationUUID);
			sb.append("</ns:ReservationId>");
			sb.append("</wsa5:ReferenceParameters>");
			sb.append("</wsa5:EndpointReference>");
			
			endpointReference = EndpointReferenceUtils.loadFromString(sb.toString());

		}
			
		return endpointReference;
	}
	
	public ReservationEndpointReference(InputStream is) throws IOException {
		this.endpointReference = EndpointReferenceUtils.loadFromStream(is);
		
		address = EndpointReferenceUtils.getElementValue(endpointReference, "Address");
		reservationUUID = EndpointReferenceUtils.getElementValue(endpointReference, "ReservationId");
		
		if (address == null || reservationUUID == null)
			throw new NullPointerException("Either Address or ReservationId is missing in given EPR");
		
	}
	
	public ReservationEndpointReference(W3CEndpointReference epr) {
		this.endpointReference = epr;
		
		address = EndpointReferenceUtils.getElementValue(epr, "Address");
		reservationUUID = EndpointReferenceUtils.getElementValue(epr, "ReservationId");
		
		if (address == null || reservationUUID == null)
			throw new NullPointerException("Either Address or ReservationId is missing in given EPR");
		
	}
	
	public ReservationEndpointReference(String address, String reservationId) {
		this.address = address;
		this.reservationUUID = reservationId;
	}
	
	public ReservationEndpointReference(String reservationId) {
		this(null, reservationId);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((reservationUUID == null) ? 0 : reservationUUID.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ReservationEndpointReference other = (ReservationEndpointReference) obj;
		if (reservationUUID == null) {
			if (other.reservationUUID != null)
				return false;
		} else if (!reservationUUID.equals(other.reservationUUID))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "ActivityEndpointReference[address=" + address + ",activityUUID=" + reservationUUID + "]";
	}

}
