package com.qcg.comp.tests;



import junit.framework.TestCase;

import com.qcg.comp.sdk.QCGFactory;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.types.ActivityEndpointReference;
import com.qcg.comp.sdk.types.QCGActivityStatus;
import com.qcg.comp.stubs.bes.factory.BasicResourceAttributesDocumentType;
import com.qcg.comp.stubs.bes.factory.FactoryResourceAttributesDocumentType;
import com.qcg.core.sdk.security.anonymous.AnonymousAuthentication;

/**
 * Unit test for simple Job submission.
 */
public class QCGCompExtendedExample 
    extends TestCase
{
	public static final String QCG_COMPUTING_HOST = "grass1.man.poznan.pl";
	public static final int QCG_COMPUTING_PORT = 19001;
	
	public void testExtended() throws Exception {
		
		/* stubs initialization */
		QCGFactory besFactory =  new QCGFactory(QCG_COMPUTING_HOST, QCG_COMPUTING_PORT,  new AnonymousAuthentication(), true);
		
		/* machines monitoring */
		FactoryResourceAttributesDocumentType cluster = besFactory.getFactoryAttributesDocument();
		
		System.out.println("Cluster Name        :" + cluster.getCommonName());
		System.out.println("Cluster description :" + cluster.getLongDescription());
		System.out.println("DRMS Type           :" + cluster.getLocalResourceManagerType());
		System.out.println("\nMachines:\n");
		for (Object rr : cluster.getContainedResource()) {
			BasicResourceAttributesDocumentType host = (BasicResourceAttributesDocumentType)rr;
			System.out.println(host.getResourceName() + "[arch=" + host.getCPUArchitecture().getCPUArchitectureName() + ",cpucount=" + host.getCPUCount() + ",physMem=" + host.getPhysicalMemory().intValue() / (1024 * 1024) + "MB]");
		}
		
		/* Job submission */
		
		/* create an JSDL document */
		JSDL dateJSDL = new JSDL("JobName");
		
		dateJSDL.setExecutable("/bin/date");
		dateJSDL.getArguments().add("-R");
		dateJSDL.getEnvironment().put("TZ", "Pacific/Honolulu");
		
		/* submit a job */
		ActivityEndpointReference jobEpr = besFactory.createActivity(dateJSDL);
		System.out.println("\nSubmitted job id: " + jobEpr.getActivityUUID());
		
		/* wait for termination */
		while (!besFactory.getActivityStatus(jobEpr).isFinalState()) {
			System.out.println("Job status: " + besFactory.getActivityStatus(jobEpr));
			Thread.sleep(1000);
		}
		
		/* terminate job */
		/* besFactory.terminateActivity(jobEpr); */
		
		/* get final status */
		QCGActivityStatus status =  besFactory.getActivityStatus(jobEpr);
		
		if (status.getBESStateName().equals(QCGActivityStatus.QCG_ACTIVITY_STATE_FINISHED)) {
			System.out.println("Job " + jobEpr.getActivityUUID() + " finished with exit status: " + status.getEndStatus().getExitStatus() );
		} else if (status.getBESStateName().equals(QCGActivityStatus.QCG_ACTIVITY_STATE_FAILED)){
			System.err.println("Job " + jobEpr.getActivityUUID() + " failed");
		} else if (status.getBESStateName().equals(QCGActivityStatus.QCG_ACTIVITY_STATE_CANCELLED)){
			System.err.println("Job " + jobEpr.getActivityUUID() + " was cancelled");
		}
		
	}
	
}
