package com.qcg.comp.tests;



import junit.framework.TestCase;

import com.qcg.comp.sdk.QCGFactory;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.comp.sdk.types.ActivityEndpointReference;
import com.qcg.comp.sdk.types.QCGActivityStatus;
import com.qcg.core.sdk.security.anonymous.AnonymousAuthentication;

/**
 * Unit test for simple Job submission.
 */
public class QCGCompSample 
    extends TestCase
{
	public static final String QCG_COMPUTING_HOST = "localhost";
	public static final int QCG_COMPUTING_PORT = 19001;
	
	public void testSubmit() throws Exception {
		QCGFactory besFactory =  new QCGFactory(QCG_COMPUTING_HOST, QCG_COMPUTING_PORT, new AnonymousAuthentication(), false);
		
		/* create an JSDL document */
		JSDL dateJSDL = new JSDL("SimpleJob");
		
		dateJSDL.setExecutable("/bin/date");
		dateJSDL.getArguments().add("-R");
		dateJSDL.getEnvironment().put("TZ", "Pacific/Honolulu");
		
		/* submit a job */
		ActivityEndpointReference jobEpr = besFactory.createActivity(dateJSDL);
		
		/* wait for termination */
		while (besFactory.getActivityStatus(jobEpr).isFinalState()) { 
			Thread.sleep(2000);
		}
		
		/* get final status */
		QCGActivityStatus status =  besFactory.getActivityStatus(jobEpr);
		
		if (status.getBESStateName().equals(QCGActivityStatus.QCG_ACTIVITY_STATE_FINISHED)) {
			System.out.println("Job " + status.getActivityIdentifier() + " finished with exit status: " + status.getEndStatus().getExitStatus() );
		} else if (status.getBESStateName().equals(QCGActivityStatus.QCG_ACTIVITY_STATE_FAILED)){
			System.err.println("Job " + status.getActivityIdentifier() + " failed");
		} else if (status.getBESStateName().equals(QCGActivityStatus.QCG_ACTIVITY_STATE_CANCELLED)){
			System.err.println("Job " + status.getActivityIdentifier() + " was cancelled");
		}
		
	}
	
}
