package com.qcg.comp.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.regex.Pattern;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.qcg.comp.sdk.QCGRsync;
import com.qcg.comp.sdk.jsdl.JSDL;
import com.qcg.core.sdk.security.AuthenticationModule;
import com.qcg.core.sdk.security.anonymous.AnonymousAuthentication;
import com.sun.xml.internal.rngom.binary.PatternBuilder;

public class RsyncStagingTest {

	protected QCGRsync rsync;

	public static final String baseDir = "/tmp";
	public static final String aDir = baseDir + "/qcg_rsync_a/";
	public static final String bDir = baseDir + "/qcg_rsync_b/";

	@Before
	public void setUp() throws Exception {
		String hostname = "grass1.man.poznan.pl";
		int port = 19003;
		boolean useWSA = true;
		AuthenticationModule am = new AnonymousAuthentication();

		rsync = new QCGRsync(hostname, port, am, useWSA);
		new File(aDir).mkdirs();
		new File(bDir).mkdirs();
	}

	private void rm(String path) {
		File f = new File(path);
		if (f.isDirectory()) {
			for (String p : f.list())
				rm(path + "/" + p);
		}
		f.delete();
	}

	@After
	public void tearDown() throws Exception {
		rm(aDir);
		rm(bDir);
	}

	protected String prepareFile(String basedir, String name, String content)
			throws FileNotFoundException {
		new File(basedir).mkdirs();
		String path = basedir + name;
		PrintStream p = new PrintStream(new FileOutputStream(path));
		p.print(content);
		p.close();
		return path;
	}

	protected String getFileContent(String path) throws IOException {
		FileInputStream f = new FileInputStream(path);
		byte[] b = new byte[(int) new File(path).length()];
		f.read(b);
		f.close();
		return new String(b);
	}

	protected void core() throws Throwable {
		JSDL jsdl = new JSDL("rsync");
		jsdl.setWorkingDirectory(aDir);
		rsync.synchronize(jsdl, bDir, null);
	}

	protected void check(String expected, String file) throws IOException {
		assertEquals(expected, getFileContent(aDir + "/" + file));
		assertEquals(expected, getFileContent(bDir + "/" + file));
	}
/*
	@Test
	public void command0x46() throws Throwable {
		StringBuffer s = new StringBuffer();
		for (int i = 0; i < 300; ++i)
			s.append('a');
		prepareFile(aDir, "1", s.toString());
		new File(prepareFile(bDir, "1", s.toString())).setLastModified(0);
		core();
		check(s.toString(), "1");
	}

	@Test
	public void command0x45() throws Throwable {
		String prefix = "aaaaaaaaaa";
		prepareFile(aDir, "1", prefix);
		new File(prepareFile(bDir, "1", prefix)).setLastModified(0);
		core();
		check(prefix, "1");
	}

	@Test
	public void command0x49() throws Throwable {
		StringBuffer aBuf = new StringBuffer();
		StringBuffer bBuf = new StringBuffer();
		for (int i = 0; i < 2048; ++i) {
			aBuf.append('a');
			bBuf.append('b');
		}
		aBuf.append("123");
		bBuf.append("123");
		prepareFile(aDir, "1", aBuf.toString());
		new File(prepareFile(bDir, "1", bBuf.toString())).setLastModified(0);
		core();
		check(aBuf.toString(), "1");
	}

	@Test
	public void command0x4a() throws Throwable {
		StringBuffer aBuf = new StringBuffer();
		StringBuffer bBuf = new StringBuffer();
		StringBuffer cBuf = new StringBuffer();
		for (int i = 0; i < 2048; ++i) {
			aBuf.append('a');
			bBuf.append('b');
			cBuf.append('c');
		}
		aBuf.append(cBuf);
		bBuf.append(cBuf);
		prepareFile(aDir, "1", aBuf.toString());
		new File(prepareFile(bDir, "1", bBuf.toString())).setLastModified(0);
		core();
		check(aBuf.toString(), "1");
	}

	@Test
	public void command0x4d() throws Throwable {
		StringBuffer aBuf = new StringBuffer();
		StringBuffer bBuf = new StringBuffer();
		for (int i = 0; i < (1 << 17); ++i) {
			aBuf.append('a');
			bBuf.append('b');
		}
		aBuf.append("12345");
		bBuf.append("12345");
		prepareFile(aDir, "1", aBuf.toString());
		new File(prepareFile(bDir, "1", bBuf.toString())).setLastModified(0);
		core();
		check(aBuf.toString(), "1");
	}

	@Test
	public void command0x4e() throws Throwable {
		StringBuffer aBuf = new StringBuffer();
		StringBuffer bBuf = new StringBuffer();
		StringBuffer cBuf = new StringBuffer();
		for (int i = 0; i < (1 << 17); ++i) {
			aBuf.append('a');
			bBuf.append('b');
		}
		for (int i = 0; i < 2048; ++i)
			cBuf.append('c');
		aBuf.append(cBuf);
		bBuf.append(cBuf);
		prepareFile(aDir, "1", aBuf.toString());
		new File(prepareFile(bDir, "1", bBuf.toString())).setLastModified(0);
		core();
		check(aBuf.toString(), "1");
	}

	@Test
	public void recv_delta_empty_file() throws Throwable {
		prepareFile(aDir, "1", "a");
		new File(prepareFile(bDir, "1", "")).setLastModified(0);
		core();
		check("a", "1");
	}

	@Test
	public void recv_delta_no_file() throws Throwable {
		prepareFile(aDir, "1", "a");
		core();
		check("a", "1");
	}

	@Test
	public void send_delta_empty_file() throws Throwable {
		prepareFile(bDir, "1", "a");
		new File(prepareFile(aDir, "1", "")).setLastModified(0);
		core();
		check("a", "1");
	}

	@Test
	public void send_delta_big_file() throws Throwable {
		StringBuilder data = new StringBuilder();
		for (int i = 0; i < 50000; ++i)
			data.append("a");
		prepareFile(bDir, "1", data.toString());
		new File(prepareFile(aDir, "1", "")).setLastModified(0);
		core();
		check(data.toString(), "1");
	}

	@Test
	public void send_delta_no_file() throws Throwable {
		prepareFile(bDir, "1", "a");
		core();
		check("a", "1");
	}

	@Test
	public void two_way() throws Throwable {
		prepareFile(aDir, "1", "1");
		prepareFile(bDir, "2", "2");
		core();
		check("1", "1");
		check("2", "2");
	}

	@Test
	public void two_way_with_dirs() throws Throwable {
		prepareFile(aDir + "/1/", "1", "1");
		prepareFile(bDir + "/2/", "2", "2");
		core();
		check("1", "1/1");
		check("2", "2/2");
	}

	@Test
	public void local_to_remote() throws Throwable {
		prepareFile(aDir, "1", "a");
		new File(prepareFile(bDir, "1", "b")).setLastModified(0);
		JSDL jsdl = new JSDL("rsync");
		jsdl.setWorkingDirectory(aDir);
		rsync.localToRemote(jsdl, bDir, null);
		check("b", "1");
	}

	@Test
	public void remote_to_local() throws Throwable {
		new File(prepareFile(aDir, "1", "b")).setLastModified(0);
		prepareFile(bDir, "1", "");
		JSDL jsdl = new JSDL("rsync");
		jsdl.setWorkingDirectory(aDir);
		rsync.remoteToLocal(jsdl, bDir, null);
		check("b", "1");
	}

	@Test
	public void filtering() throws Throwable {
		prepareFile(bDir, "1", "a");
		prepareFile(bDir, "2", "a");

		ArrayList<Pattern> exclude = new ArrayList<Pattern>();
		exclude.add(Pattern.compile("1"));

		JSDL jsdl = new JSDL("rsync");
		jsdl.setWorkingDirectory(aDir);
		rsync.localToRemote(jsdl, bDir, exclude);
		check("a", "2");
		assertFalse(new File(aDir + "/1").exists());
	}
	*/
	@Test
	public void localAndRemoteTest() throws Throwable {
		prepareFile("/tmp/rsync-local-to-remote/", "1", "a");
		prepareFile("/tmp/rsync-local-to-remote/", "2", "b");
		
		JSDL jsdl = new JSDL("rsync");
		jsdl.setWorkingDirectory("/home/jkonczak/rtest");
		
		//rsync.localToRemote(jsdl, "/tmp/rsync-local-to-remote/", null);
		ArrayList<Pattern> pattern = new ArrayList<Pattern>();
		pattern.add(Pattern.compile("\\..*"));
		System.err.println(jsdl);
		System.err.println("syncing with: " + jsdl.getWorkingDirectory());
		rsync.localToRemote(jsdl, "/tmp/rtest", pattern);
		rsync.remoteToLocal(jsdl, "/tmp/rtest2", pattern);

		
		assertEquals("a", getFileContent("/tmp/rsync-remote-to-local/" + "1"));
		assertEquals("b", getFileContent("/tmp/rsync-remote-to-local/" + "2"));


	}
}
