dnl $Id: qcg-core.m4 2763 2009-10-05 10:05:25Z mmamonski $

dnl
dnl Copyright 2006-2009 QCG Project Team
dnl
dnl Licensed under the Apache License, Version 2.0 (the "License");
dnl you may not use this file except in compliance with the License.
dnl You may obtain a copy of the License at
dnl
dnl     http://www.apache.org/licenses/LICENSE-2.0
dnl
dnl Unless required by applicable law or agreed to in writing, software
dnl distributed under the License is distributed on an "AS IS" BASIS,
dnl WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
dnl See the License for the specific language governing permissions and
dnl limitations under the License.
dnl

dnl AM_PATH_QCG_CORE([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])

AC_DEFUN([AM_PATH_QCG_CORE],[ 
AC_ARG_WITH(qcg-core,
  AS_HELP_STRING([--with-qcg-core=PATH], [Prefix (e.g. /usr/local) where QCG Core is installed (optional)]),
  qcg_core_prefix="$withval", qcg_core_prefix="")

AC_ARG_ENABLE(qcg-core-test,
  AS_HELP_STRING([--disable-qcg-core-test], [Do not try to compile and run a test program]),,
  enable_qcg_core_test=yes)

  if test x$qcg_core_prefix != x; then
    if test -x "$qcg_core_prefix/bin/qcg-core-config"; then
      QCG_CORE_CONFIG=$qcg_core_prefix/bin/qcg-core-config
    fi
  fi

  AC_PATH_PROG(QCG_CORE_CONFIG, qcg-core-config, no, [${PATH}:/opt/qcg/bin])
  min_qcg_core_version=ifelse([$1],, 2.0.0, [$1])
  AC_MSG_CHECKING(for QCG Core - version >= $min_qcg_core_version)
  no_qcg_core=""
  if test "$QCG_CORE_CONFIG" = "no"; then
    no_qcg_core=yes
  else
    QCG_CORE_PREFIX=`$QCG_CORE_CONFIG -b`
    QCG_CORE_INCLUDE=`$QCG_CORE_CONFIG -i`
    QCG_CORE_CPPFLAGS=`$QCG_CORE_CONFIG -c`
    QCG_CORE_LIBS=`$QCG_CORE_CONFIG -l`
    QCG_CORE_TRANSPORT_LIBS=`$QCG_CORE_CONFIG -k`
    QCG_CORE_DIR_XSD=`$QCG_CORE_CONFIG -x`
    QCG_CORE_DIR_MODULEEXEC=`$QCG_CORE_CONFIG -m`
    QCG_CORE_AUTH_MECH=`$QCG_CORE_CONFIG -t`
    qcg_core_config_major_version=`$QCG_CORE_CONFIG -v | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    qcg_core_config_minor_version=`$QCG_CORE_CONFIG -v | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    qcg_core_config_micro_version=`$QCG_CORE_CONFIG -v | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_qcg_core_test" = "xyes"; then
      ac_save_CPPFLAGS="$CPPFLAGS"
      ac_save_LIBS="$LIBS"
      CPPFLAGS="$CPPFLAGS $QCG_CORE_CPPFLAGS"
      LIBS="$QCG_CORE_LIBS $LIBS"

dnl
dnl Now check if the installed qcg-core is sufficiently new.
dnl (Also sanity checks the results of qcg-core-config to some extent)
dnl
      rm -f conf.qcgcoretest
      AC_RUN_IFELSE([AC_LANG_SOURCE([[
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qcg-core/version.h>

int 
main()
{
  int qcg_core_major_version, qcg_core_minor_version, qcg_core_micro_version;
  int major, minor, micro;
  char *tmp_version;

  system("touch conf.qcgcoretest");

  tmp_version = strdup("$min_qcg_core_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3)
    return 1;
  free(tmp_version);

  /* Compare qcg-core-config output to the qcg-core headers */
  if ((QCG_CORE_VERSION_MAJOR != $qcg_core_config_major_version) ||
      (QCG_CORE_VERSION_MINOR != $qcg_core_config_minor_version) ||
      (QCG_CORE_VERSION_MICRO != $qcg_core_config_micro_version)
      )
    {
      printf("*** qcg-core header files do not match\n");
      return 1;
    } 

  qcg_core_major_version = QCG_CORE_VERSION_MAJOR;
  qcg_core_minor_version = QCG_CORE_VERSION_MINOR;
  qcg_core_micro_version = QCG_CORE_VERSION_MICRO;

  /* Test that the library is greater than our minimum version */
  if ((qcg_core_major_version == major && qcg_core_minor_version > minor) ||
	  (qcg_core_major_version == major && qcg_core_minor_version == minor && qcg_core_micro_version >= micro))
  {
    return 0;
  } else if (qcg_core_major_version > major) {
    printf("\n*** An incompatible version of qcg-core (%d.%d.%d) was found.\n", qcg_core_major_version, qcg_core_minor_version, qcg_core_micro_version);
    printf("*** You need a version of qcg-core newer than %d.%d.%d but within %d.x.x major version.\n", major, minor, micro, major);
    return 1;
  } else {
    printf("\n*** An old version of qcg-core (%d.%d.%d) was found.\n", qcg_core_major_version, qcg_core_minor_version, qcg_core_micro_version);
    printf("*** You need a version of qcg-core newer than %d.%d.%d.\n", major, minor, micro);
    return 1;
  }
}
]])],[],[no_qcg_core=yes],[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CPPFLAGS="$ac_save_CPPFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi

  if test "x$no_qcg_core" = x ; then
     AC_MSG_RESULT(yes (version $qcg_core_config_major_version.$qcg_core_config_minor_version.$qcg_core_config_micro_version))
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$QCG_CORE_CONFIG" = "no" ; then
       echo "*** The qcg-core-config script installed by qcg-core could not be found"
       echo "*** If qcg-core was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the QCG_CORE_CONFIG environment variable to the"
       echo "*** full path to qcg-core-config."
     else
       if test -f conf.qcgcoretest ; then
        :
       else
          echo "*** Could not run qcg-core test program, checking why..."
          CPPFLAGS="$CPPFLAGS $QCG_CORE_CPPFLAGS"
          LIBS="$LIBS $QCG_CORE_LIBS"
          AC_LINK_IFELSE([AC_LANG_PROGRAM([[
#include <qcg-core/sm.h>
#include <stdio.h>
]], [[ LIBQCG_CORE_TEST_VERSION; return 0;]])],[ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding qcg-core or finding the wrong"
          echo "*** version of qcg-core. If it is not finding qcg-core, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
          echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH" ],[ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means qcg-core was incorrectly installed"
          echo "*** or that you have moved qcg-core since it was installed." ])
          CPPFLAGS="$ac_save_CPPFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi

     QCG_CORE_CPPFLAGS=""
     QCG_CORE_LIBS=""
     QCG_CORE_DIR_XSD=""
     QCG_CORE_PREFIX=""
     QCG_CORE_INCLUDE=""
     QCG_CORE_DIR_MODULEEXEC=""
     QCG_CORE_TRANSPORT_LIBS=""
     QCG_CORE_AUTH_MECH=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(QCG_CORE_CPPFLAGS)
  AC_SUBST(QCG_CORE_LIBS)
  AC_SUBST(QCG_CORE_DIR_XSD)
  AC_SUBST(QCG_CORE_DIR_MODULEEXEC)
  AC_SUBST(QCG_CORE_PREFIX)
  AC_SUBST(QCG_CORE_INCLUDE)
  AC_SUBST(QCG_CORE_TRANSPORT_LIBS)
  AC_SUBST(QCG_CORE_AUTH_MECH)
  rm -f conf.qcgcoretest
])
