/* $Id: python_module.c 2 2009-10-05 10:09:52Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string.h>
#include <stdlib.h>

#include <qcg-core/sm.h>
#include <qcg-core/python.h>
#include <qcg-core/logger.h>
#include <qcg-core/modules/modules.h>

#include <common/python_module.h>
#include <include/extensions.h>
#include <include/ardl_filter.h>
#include <include/jsdl_filter.h>
#include <include/namespaces.h>

#include <qcg-core-config.h>

#ifdef WITH_PYTHON
#  include <Python.h>
#  include <structmember.h>
#endif

static sm_status_ret_code_t sm_python_internal_stuff(void);

sm_status_ret_code_t
smc_python_setup(void)
{
#ifndef WITH_PYTHON
	return SM_OK;
#else
	char buf[4096];

	SM_DEBUG_ENTER();

	SM_TRY_FAULT(sm_python_setup());

	SM_TRY_FAULT(sm_python_internal_stuff());

	snprintf(buf, sizeof(buf),
	"JSDL_FILTER_OK = %d\n"
	"JSDL_FILTER_INTERNAL_FAULT = %d\n"
	"JSDL_FILTER_UNSUPPORTED_FEATURE_FAULT = %d\n"
	"JSDL_FILTER_INVALID_REQUEST_MESSAGE_FAULT = %d\n"
	"ARDL_FILTER_OK = %d\n"
	"ARDL_FILTER_INTERNAL_FAULT = %d\n"
	"ARDL_FILTER_UNSUPPORTED_FEATURE_FAULT = %d\n"
	"ARDL_FILTER_INVALID_REQUEST_MESSAGE_FAULT = %d\n"
	"SMC_NAMESPACE_CONFIG = \"%s\"\n"
	"SMC_NAMESPACE_SERVICE = \"%s\"\n"
	"SMC_NAMESPACE_FACTORY = \"%s\"\n"
	"SMC_NAMESPACE_ARDL = \"%s\"\n"
	"SMC_NAMESPACE_ARES = \"%s\"\n"
	,
	JSDL_FILTER_OK,
	JSDL_FILTER_INTERNAL_FAULT,
	JSDL_FILTER_UNSUPPORTED_FEATURE_FAULT,
	JSDL_FILTER_INVALID_REQUEST_MESSAGE_FAULT,
	ARDL_FILTER_OK,
	ARDL_FILTER_INTERNAL_FAULT,
	ARDL_FILTER_UNSUPPORTED_FEATURE_FAULT,
	ARDL_FILTER_INVALID_REQUEST_MESSAGE_FAULT,
	SMC_NAMESPACE_CONFIG,
	SMC_NAMESPACE_SERVICE,
	SMC_NAMESPACE_FACTORY,
	SMC_NAMESPACE_ARDL,
	SMC_NAMESPACE_ARES
	);

	if (sm_python_load_module_code_from_string("qcg.comp", buf) != SM_OK) {
		sm_error("Failed to extend Python module");
		goto fault;
	}

	return SM_OK;

fault:
	return SM_FAULT;
#endif
}

#ifdef WITH_PYTHON
/* There's no such API in fedcore so we have to do all this by hand here... */

static PyObject *
smc_extensions_add_python(PyObject *self sm_unused, PyObject *args)
{
	const char *uri = NULL;

	SM_DEBUG_ENTER();

	if (!PyArg_ParseTuple(args, "s", &uri))
		return NULL;

	if (smc_extensions_add(uri) != SM_OK) {
		PyErr_Format(PyExc_Exception, "Failed to register a BES Extension");
		return NULL;
	}

	return Py_BuildValue("");
}

static PyMethodDef qcg_comp_methods[] = {
	{ "extensions_add", smc_extensions_add_python, METH_VARARGS, "Register a BES Extension." },
	{ NULL, NULL, 0, NULL }
};

static sm_status_ret_code_t
sm_python_internal_stuff(void)
{
	PyObject *module;

	SM_DEBUG_ENTER();

	if (!(module = Py_InitModule("qcg.comp", qcg_comp_methods)))
		return SM_FAULT;

	return SM_OK;
}
#endif
