#include <decaf/lang/Thread.h>
#include <decaf/lang/Runnable.h>
#include <decaf/util/concurrent/CountDownLatch.h>
#include <activemq/core/ActiveMQConnectionFactory.h>
#include <activemq/core/ActiveMQConnection.h>
#include <activemq/transport/DefaultTransportListener.h>
#include <activemq/library/ActiveMQCPP.h>
#include <decaf/lang/Integer.h>
#include <activemq/util/Config.h>
#include <decaf/util/Date.h>
#include <cms/Connection.h>
#include <cms/Session.h>
#include <cms/TextMessage.h>
#include <cms/BytesMessage.h>
#include <cms/MapMessage.h>
#include <cms/ExceptionListener.h>
#include <cms/MessageListener.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>

using namespace activemq;
using namespace activemq::core;
using namespace activemq::transport;
using namespace decaf::lang;
using namespace decaf::util;
using namespace decaf::util::concurrent;
using namespace cms;
using namespace std;

class ServerActivemq : public ExceptionListener,
                            public MessageListener,
                            public DefaultTransportListener {

	private: 
		Connection* connection;
		Session* session;
		Destination* destination;
		MessageConsumer* consumer;
		
	public:
	
		void connect(char  *service_address, char *queue_name)
		{
			printf("============ Connect ==========\n");
			std::string brokerURI = "failover:(" +(string) service_address+ ")";
			cout << brokerURI << endl;	
			
			activemq::library::ActiveMQCPP::initializeLibrary();
			try 
			{
				// Create a ConnectionFactory
				ActiveMQConnectionFactory* connectionFactory = new ActiveMQConnectionFactory( brokerURI );
				
				// Create a Connection
				connection = connectionFactory->createConnection();
				delete connectionFactory;
				
				ActiveMQConnection* amqConnection = dynamic_cast<ActiveMQConnection*>( connection );
				if( amqConnection != NULL ) {
					amqConnection->addTransportListener( this );
				}

				connection->start();
				
				connection->setExceptionListener(this);
				
				session = connection->createSession( Session::AUTO_ACKNOWLEDGE );

				destination = session->createQueue( (string) queue_name );

				// Create a MessageConsumer from the Session to the Topic or Queue
				consumer = session->createConsumer( destination );
				consumer->setMessageListener( this );

			} catch (CMSException& e) {
					e.printStackTrace();
			}
		}
		
		virtual ~ServerActivemq(){
		}

		void close() {
			this->disconnect();
		}

		// Called from the consumer since this class is a registered MessageListener.
		virtual void onMessage( const Message* message )
		{
			static int count = 0;

			try
			{
				count++;
				const TextMessage* textMessage =
					dynamic_cast< const TextMessage* >( message );
				string text = "";

				if( textMessage != NULL ) {
					text = textMessage->getText();
				} else {
					text = "NOT A TEXTMESSAGE!";
				}

				printf( "Message #%d Received: %s\n", count, text.c_str() );
			} catch (CMSException& e) {
				e.printStackTrace();
			}
		}

		virtual void onException( const CMSException& ex AMQCPP_UNUSED ) {
			printf("CMS Exception occurred.  Shutting down client.\n");
			exit(1);
		}

		virtual void transportInterrupted() {
			std::cout << "The Connection's Transport has been Interrupted." << std::endl;
		}

		virtual void transportResumed() {
			std::cout << "The Connection's Transport has been Restored." << std::endl;
		}
		
	private:
		void disconnect()
		{
			printf("============ Disonnect ==========\n");
			try{
				if( destination != NULL ) delete destination;
			}catch (CMSException& e) {}
			destination = NULL;

			try{
				if( consumer != NULL ) delete consumer;
			}catch (CMSException& e) {}
			consumer = NULL;

			try{
				if( session != NULL ) session->close();
				if( connection != NULL ) connection->close();
			}catch (CMSException& e) {}

			try{
				if( session != NULL ) delete session;
			}catch (CMSException& e) {}
			session = NULL;

			try{
				if( connection != NULL ) delete connection;
			}catch (CMSException& e) {}
			connection = NULL;

			activemq::library::ActiveMQCPP::shutdownLibrary();
		}
};

int main(int argc, char ** argv)
{
	printf("============ Program start ============\n");
 	ServerActivemq server;
	server.connect((char *) "tcp://127.0.0.1:61616", (char *)"test-jobs");
	
	// Wait to exit.
    	std::cout << "Press 'q' to quit" << std::endl;
   	while( std::cin.get() != 'q') {}
	
	server.close();
	printf("============ Program end =============\n");	
	return 0;
}
