#include <activemqcpp.h>

#include <decaf/lang/Thread.h>
#include <decaf/lang/Runnable.h>
#include <decaf/util/concurrent/CountDownLatch.h>
#include <decaf/lang/Long.h>
#include <decaf/util/Date.h>
#include <activemq/core/ActiveMQConnectionFactory.h>
#include <activemq/util/Config.h>
#include <activemq/library/ActiveMQCPP.h>
#include <cms/Connection.h>
#include <cms/Session.h>
#include <cms/TextMessage.h>
#include <cms/BytesMessage.h>
#include <cms/MapMessage.h>
#include <cms/ExceptionListener.h>
#include <cms/MessageListener.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <memory>

using namespace activemq;
using namespace activemq::core;
using namespace decaf;
using namespace decaf::lang;
using namespace decaf::util;
using namespace decaf::util::concurrent;
using namespace cms;
using namespace std;

static Connection* connection;
static Session* session;
static Destination* destination;
static MessageProducer* producer;

static int cleanup(void);

int activemqcpp_connect(const char  *service_address, const char *queue_name)
{
	if(cleanup() == 1 ) /* Sometimes disconnect is not possible so we have to do connect after connect -> cleanup() first */
		return 1;
	
	std::string brokerURI = (string) service_address;
	activemq::library::ActiveMQCPP::initializeLibrary();
	try 
	{
		auto_ptr<ActiveMQConnectionFactory> connectionFactory(new ActiveMQConnectionFactory( brokerURI  ) );
		try
		{
			connection = connectionFactory->createConnection();
			connection->start();
		}
		catch( CMSException& e ) 
		{
			return 1;
		}

		session = connection->createSession( Session::AUTO_ACKNOWLEDGE );
		destination = session->createQueue( (string) queue_name );
		producer = session->createProducer( destination);

	}
	catch ( CMSException& e ) 
	{
		e.printStackTrace();
		return 1;
	}
	return 0;
}

int activemqcpp_send_message(const char *raw_xml)
{
	try
	{
		string text = (string) raw_xml;
		TextMessage* message = session->createTextMessage( text );
		producer->send( message );
		delete message;
	}
	catch ( CMSException& e ) 
	{
		e.printStackTrace();
		return 1;
	}

	return 0;
}


int activemqcpp_disconnect()
{
	if(cleanup() ==1 )
		return 1;
	
	activemq::library::ActiveMQCPP::shutdownLibrary();
	return 0;
}

int cleanup()
{	
	try
	{
		if( destination != NULL ) 
		{
			delete destination;
			destination = NULL;
		}
	} catch ( CMSException& e ) 
	{ 
		e.printStackTrace(); 
		return 1; 
	}
	destination = NULL;

	try
	{
		if( producer != NULL ) 
		{
			delete producer;
			producer = NULL;
		}
	} catch ( CMSException& e ) 
	{ 
		e.printStackTrace(); 
		return 1; 
	}

	try
	{
		if( session != NULL ) 
			session->close();
		if( connection != NULL ) 
			connection->close();
	} catch ( CMSException& e ) 
	{ 
		e.printStackTrace(); 
		return 1;
	}

	try
	{
		if( session != NULL ) 
		{
			delete session;
			session = NULL;
		}
	} catch ( CMSException& e ) 
	{ 
		e.printStackTrace(); 
		return 1;
	}

	try {
		if( connection != NULL ) 
		{
			delete connection;
			connection = NULL;
		}
	} catch ( CMSException& e ) 
	{ 
		e.printStackTrace(); 
		return 1;
	}

	return 0;
}
