# $Id: InfoStorage.py 73 2010-03-13 01:36:13Z versor $

#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at

#      http://www.apache.org/licenses/LICENSE-2.0

#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import datetime

class ReservationInfo(object):
    "Reservation information storage class"
    
    __TYPE = [ "ar", "sr", "job" ]
    
    def __init__(self):
        self.__reservation_name = ""
        self.__type = "Unknown"
        self.__start_time = None
        self.__end_time = None
        self.__owner = ""
        self.__resources = dict()
        
    def __getReservationName(self):
        return self.__reservation_name
    
    def __setReservationName(self, value):
        if not isinstance(value, str):
            raise TypeError("ReservationName must be a string")
        self.__reservation_name = value
        
    def __getType(self):
        return self.__type
    
    def __setType(self, value):
        if not isinstance(value, str):
            raise TypeError("ReservationName must be a string")
        if not value in ReservationInfo.__TYPE:
            raise ValueError("Invalid reservation type")
        self.__type = value  
        
    def __getStartTime(self):
        return self.__start_time
    
    def __setStartTime(self, value):
        if not isinstance(value, datetime.datetime):
            raise TypeError("StartTime must be datetime object")
        self.__start_time = value
        
    def __getEndTime(self):
        return self.__end_time
    
    def __setEndTime(self, value):
        if not isinstance(value, datetime.datetime):
            raise TypeError("EndTime must be datetime object")
        if self.__start_time != None and self.__start_time > value:
            raise ValueError("EndTime must be later than StartTime")
        self.__end_time = value
        
    def __getOwner(self):
        return self.__owner
    
    def __setOwner(self, value):
        if not isinstance(value, str):
            raise TypeError("Owner must be a string")
        self.__owner = value
        
    def __getResources(self):
        return self.__resources
    
    def __setResources(self, value):
        if not isinstance(value, dict):
            raise TypeError("Resources must be a dict")
        for key in value.keys():
            if not isinstance(key, str):
                raise TypeError("Keys must be a string")
            if not isinstance(value[key], int):
                raise TypeError("Values must be a integer")
        self.__resources = value
        
    ReservationName = property(__getReservationName, __setReservationName, None, \
                               "String, gets/sets reservation name")
    
    Type = property(__getType, __setType, None, \
                    "String, gets/sets reservation type")
    
    StartTime = property(__getStartTime, __setStartTime, None, \
                         "DateTime, gets/sets reservation start time")
    
    EndTime = property(__getEndTime, __setEndTime, None, \
                       "DateTime, gets/sets reservation end time")
    
    Owner = property(__getOwner, __setOwner, None, \
                     "String, gets/sets reservation owner name")
    
    Resources = property(__getResources, __setResources, None, \
                         "Dict, gets/sets dictionary containings resources name (as keys) and resource count (as values)")
        

class HostInfo(object):
    "Host information storage class"
    
    __OS = [ "Unknown", "Winnt", "Solaris", "Linux", "LINUX" ]
    __CPU_ARCH = [ "Other", "ARM", "IA-64", "MIPS", "RISC", "SPARC", "x86", "x86_64"]
            
    def __init__(self):                
        self.__host_name = "" 
        self.__cpu_arch = "Unknown"
        self.__os_name = "Unknown"
        self.__cpu_speed = 0.0
        self.__cpu_count = 1
        self.__cpu_utilization = None
        self.__15s_queue_lenght = None
        self.__1m_queue_lenght = None
        self.__15m_queue_lenght = None       
        self.__paging_activity = None
        self.__job_num = None
        self.__job_max = None
        self.__free_mem = None
        self.__free_swap = None
        self.__free_fs = None
        self.__total_mem = None
        self.__total_swap = None
        self.__total_fs = None
        self.__queues = []
        self.__jobs = []
        self.__additional_resources = dict()    
    
    def __getHostName(self):        
        return self.__host_name
        
    def __setHostName(self, value):
        if not isinstance(value, str):
            raise TypeError("HostName must be a string")
        self.__host_name = value
        
    def __getCpuArch(self):        
        return self.__cpu_arch
        
    def __setCpuArch(self, value):
        if value == None:
            self.__cpu_arch = None
            return
        if not isinstance(value, str):
            raise TypeError("Architecture name must be a string")        
        if not value in HostInfo.__CPU_ARCH:
            raise ValueError("Invalid architecture name")    
        self.__cpu_arch = value
    
    def __getOsName(self):        
        return self.__os_name
        
    def __setOsName(self, value):
        if value == None:
            self.__cpu_arch = None
            return
        if not isinstance(value, str):
            raise TypeError("OS name must be a string")
        value = value.capitalize()
        if not value in HostInfo.__OS:
            raise ValueError("Invalid OS name")    
        self.__os_name = value

    def __getCpuSpeed(self):        
        return self.__cpu_speed

    def __setCpuSpeed(self, value):
        if not isinstance(value, float):
            raise TypeError("CPU speed must be a float")
        if value < 0.0:
            raise ValueError("CPU speed must be greater than 0")
        self.__cpu_speed = value 
        
    def __getCpuCount(self):        
        return self.__cpu_count

    def __setCpuCount(self, value):
        if not isinstance(value, int):
            raise TypeError("CPU count must be an integer")
        if value < 1:
            raise ValueError("CPU count must be greater than 0")
        self.__cpu_count = value        
            
    def __getCpuUtilization(self):        
        return self.__cpu_utilization
    
    def __setCpuUtilization(self, value):
        if value == None:
            self.__cpu_utilization = value
            return
        if not isinstance(value, float):
            raise TypeError("CpuUtilization should be a float")
        if value < 0 or value > 100:
            raise ValueError("CpuUtilization must be in 0-100 range")
        self.__cpu_utilization = value    
    
    def __getQueueLenght15S(self):        
        return self.__15s_queue_lenght
    
    def __setQueueLenght15S(self, value):
        if not isinstance(value, float):
            raise TypeError("QueueLenght15S should be a float")
        if value < 0:
            raise ValueError("QueueLenght15S should be positive number")                
        self.__15s_queue_lenght = value
        
    def __getQueueLenght1M(self):        
        return self.__1m_queue_lenght
    
    def __setQueueLenght1M(self, value):
        if not isinstance(value, float):
            raise TypeError("QueueLenght1M should be a float")
        if value < 0:
            raise ValueError("QueueLenght1M should be positive number")                
        self.__1m_queue_lenght = value
        
    def __getQueueLenght15M(self):        
        return self.__15m_queue_lenght
    
    def __setQueueLenght15M(self, value):
        if not isinstance(value, float):
            raise TypeError("QueueLenght15M should be a float")
        if value < 0:
            raise ValueError("QueueLenght15M should be positive number")                
        self.__15m_queue_lenght = value
        
    def __getPagingActivity(self):
        return self.__paging_activity
    
    def __setPagingActivity(self, value):
        if not isinstance(value, float):
            raise TypeError("PagingActivity should be float")
        if value<0:
            raise ValueError("PagingActivity should be positive number")
        self.__paging_activity = value

    def __getJobNum(self):
        return self.__job_num
    
    def __setJobNum(self, value):
        if not isinstance(value, int):
            raise TypeError("JobNum should be int")
        if value<0:
            raise ValueError("JobNum should be positive number")
        self.__job_num = value

    def __getJobMax(self):
        return self.__job_max
    
    def __setJobMax(self, value):
        if not isinstance(value, int):
            raise TypeError("JobMax should be int")
        if value<0:
            raise ValueError("JobMax should be positive number")
        self.__job_max = value
        
    def __getFreeMemory(self):
        return self.__free_mem
    
    def __setFreeMemory(self, value):
        if not isinstance(value, float):
            raise TypeError("FreeMemory should be float")
        if value<0:
            raise ValueError("FreeMemory should be positive number")
        self.__free_mem = value
        
    def __getFreeSwap(self):
        return self.__free_swap
    
    def __setFreeSwap(self, value):
        if not isinstance(value, float):
            raise TypeError("FreeSwap should be float")
        if value<0:
            raise ValueError("FreeSwap should be positive number")
        self.__free_swap = value
        
    def __getFreeFileSystem(self):
        return self.__free_fs
    
    def __setFreeFileSystem(self, value):
        if not isinstance(value, float):
            raise TypeError("FreeFileSystem should be float")
        if value<0:
            raise ValueError("FreeFileSystem should be positive number")
        self.__free_fs = value
        
    def __getTotalMemory(self):
        return self.__total_mem
    
    def __setTotalMemory(self, value):
        if not isinstance(value, float):
            raise TypeError("TotalMemory should be float")
        if value<0:
            raise ValueError("TotalMemory should be positive number")
        self.__total_mem = value
        
    def __getTotalSwap(self):
        return self.__total_swap
    
    def __setTotalSwap(self, value):
        if not isinstance(value, float):
            raise TypeError("TotalSwap should be float")
        if value<0:
            raise ValueError("TotalSwap should be positive number")
        self.__total_swap = value
        
    def __getTotalFileSystem(self):
        return self.__total_fs
    
    def __setTotalFileSystem(self, value):
        if not isinstance(value, float):
            raise TypeError("TotalFileSystem should be float")
        if value<0:
            raise ValueError("TotalFileSystem should be positive number")
        self.__total_fs = value
        
    def __getQueues(self):
        return self.__queues
    
    def __setQueues(self, value):
        if not isinstance(value, list):
            raise TypeError("Queues must be a list")
        self.__queues = value
        
    def __getJobs(self):
        return self.__jobs
    
    def __setJobs(self, value):
        if not isinstance(value, list):
            raise TypeError("Jobs must be a list")
        self.__jobs = value
    
    def __getAdditionalResources(self):
        return self.__additional_resources
    
    def __setAdditionalResources(self, value):
        if not isinstance(value, dict):
            raise TypeError("AdditionalResources must be a dictionary")
        self.__additional_resources = value
          
        
    HostName = property(__getHostName, __setHostName, None, "Gets/sets host name")
    CpuArch = property(__getCpuArch, __setCpuArch, None, "Gets/sets CPU architecture name, valid values are\n\
    Unknown,\n\
    ARM,\n\
    IA-64,\n\
    MIPS,\n\
    RISC,\n\
    SPARC,\n\
    x86")
    
    OSName = property(__getOsName, __setOsName, None, \
    "Gets OS name. Valid values are:\n\
    Unknown,\n\
    Windows,\n\
    Solaris,\n\
    Linux")
    CpuCount = property(__getCpuCount, __setCpuCount, None, "Integer, gets/sets CPU count")
    
    CpuSpeed = property(__getCpuSpeed, __setCpuSpeed, None, "Integer, gets/sets CPU speed")
    
    CpuUtilization = property(__getCpuUtilization, __setCpuUtilization, None, \
                              "Float, gets/sets CPU utilization (average over minute) in percents")
    QueueLength15S = property(__getQueueLenght15S, __setQueueLenght15S, None, \
                              "Float, gets/sets average queue length over past 15 seconds")
    QueueLength1M = property(__getQueueLenght1M, __setQueueLenght1M, None, \
                              "Float, gets/sets average queue length over past minute")
    QueueLength15M = property(__getQueueLenght15M, __setQueueLenght15M, None, \
                              "Float, gets/sets average queue length over past 15 minutes")
    PagingActivity = property(__getPagingActivity, __setPagingActivity, None, \
                              "Float, gets/sets number of pages in/out per second")
    JobNum = property(__getJobNum, __setJobNum, None, \
                       "Int, gets/sets number of jobs currently running in system")
    JobMax = property(__getJobMax, __setJobMax, None, \
                       "Int, gets/sets maximum number of jobs that can be running simultaneously")
    FreeMemory = property(__getFreeMemory, __setFreeMemory, None, \
                       "Float, gets/sets free memory on the host in MB")
    FreeSwap = property(__getFreeSwap, __setFreeSwap, None, \
                        "Float, gets/sets free swap space on the host in MB")
    FreeFilesystem = property(__getFreeFileSystem, __setFreeFileSystem, None, \
                              "Float, gets/sets free space on the host file system in MB")
    TotalMemory = property(__getTotalMemory, __setTotalMemory, None, \
                       "Float, gets/sets total memory on the host in MB")
    TotalSwap = property(__getTotalSwap, __setTotalSwap, None, \
                        "Float, gets/sets total swap space on the host in MB")
    TotalFilesystem = property(__getTotalFileSystem, __setTotalFileSystem, None, \
                              "Float, gets/sets total space on the host file system in MB")
    Jobs = property(__getJobs, __setJobs, None, \
                    "List, gets/sets jobs currently scheduled on the host")
    Queues = property(__getQueues, __setQueues, None, \
                      "List, gets/sets queues for whom host is currently available")
    AdditionalResources = property(__getAdditionalResources, __setAdditionalResources, None, \
                        "Dict, gets/sets any additional resources on the host")    

class QueueInfo(object):
    "Host information storage class"
        
    def __init__(self):        
        self.__queue_name = ""
        self.__queue_priority = 0
        self.__queue_active = False
        self.__queue_open = False
        self.__pending_jobs = 0
        self.__running_jobs = 0
        self.__max_jobs = 0        
        self.__job_ids = []
        self.__additional_info = dict()
        self.__is_default = False
        self.__walltime = None
        
    def __getQueueName(self):
        return self.__queue_name
    
    def __setQueueName(self, value):
        if not isinstance(value, str):
            raise TypeError("QueueName should be a string")
        self.__queue_name = value
        
    def __getQueuePriority(self):
        return self.__queue_priority
    
    def __setQueuePriority(self,value):
        if not isinstance(value, int):
            raise TypeError("QueuePriority should be an integer")
        self.__queue_priority = value
        
    def __getQueueEnabled(self):
        return self.__queue_active
    
    def __setQueueEnabled(self,value):
        if not isinstance(value, bool):
            raise TypeError("QueueActive should be a bool value")
        self.__queue_active = value    
    
    def __getPendingJobs(self):
        return self.__pending_jobs
    
    def __setPendingJobs(self,value):
        if not isinstance(value, int):
            raise TypeError("PendingJobs should be an integer")
        self.__pending_jobs = value
        
    def __getRunningJobs(self):
        return self.__running_jobs
    
    def __setRunningJobs(self,value):
        if not isinstance(value, int):
            raise TypeError("RunningJobs should be an integer")
        self.__running_jobs = value
        
    def __getMaxJobs(self):
        return self.__max_jobs
    
    def __setMaxJobs(self,value):
        if not isinstance(value, int):
            raise TypeError("MaxJobs should be an integer")
        self.__max_jobs = value
        
    def __getJobIDs(self):
        return self.__job_ids
    
    def __setJobIDs(self, value):
        if not isinstance(value, list):
            raise TypeError("JobIDs should be a list")
        self.__job_ids = value
        
    def __getAdditionalInfo(self):
        return self.__additional_info
    
    def __setAdditionalInfo(self,value):
        if not isinstance(value, dict):
            raise TypeError("AdditionalInfo should be a dictionary")
        self.__additional_info = value
        
    def __getIsDefault(self):
        return self.__is_default
    
    def __setIsDefault(self, value):
        if not isinstance(value, bool):
            raise TypeError("IsDefault should be bool")
        self.__is_default = value
        
    def __getWalltime(self):
        return self.__walltime
    
    def __setWalltime(self, value):
        if not isinstance(value, datetime.timedelta):
            raise TypeError("Walltime should be timedelta instance.")
        self.__walltime = value            
        
    QueueName = property(__getQueueName, __setQueueName, None, \
                         "string, gets/sets queue name")
    QueuePriority = property(__getQueuePriority, __setQueuePriority, None, \
                             "integer, gets/sets queue priority level")
    QueueEnabled = property(__getQueueEnabled, __setQueueEnabled, None, \
                           "bool, gets/sets if queue is enabled (dispatches & runs it jobs)")
    PendingJobs = property(__getPendingJobs, __setPendingJobs, None, \
                           "integer, gets/sets count of pending jobs in the queue")
    RunningJobs = property(__getRunningJobs, __setRunningJobs, None, \
                           "integer, gets/sets count of running jobs in the queue")
    MaxJobs = property(__getMaxJobs, __setMaxJobs, None, \
                       "integer, gets/sets maximum count of jobs in the queue")
    JobIDs = property(__getJobIDs, __setJobIDs, None,\
                      "list, gets/sets list of all jobs currently in the queue")
    AdditionalInfo = property(__getAdditionalInfo, __setAdditionalInfo, None,\
                              "dictionary, gets/sets dictionary of all additional information about queue")
    IsDefault = property(__getIsDefault, __setIsDefault, None,\
                         "bool, gets/sets value indicating if queue is default")
    Walltime = property(__getWalltime, __setWalltime, None,\
                        "timedelta, gets/sets value of the walltime")

        
class JobInfo(object):
    "Job information storage class"
    
    __STATUS = [ "Pending", "Running", "Suspended", "Done", "Exited", "Unknown" ]
    
    def __init__(self):        
        self.__job_id = ""
        self.__job_name = ""
        self.__owner_name = ""
        self.__job_status = ""
        self.__queue_name = ""
        self.__submission_time = None
        self.__start_time = None
        self.__end_time = None
        self.__execution_hosts = []
        self.__total_mem = 0
        self.__total_swap = 0
        self.__user_time = 0
        self.__system_time = 0
        
    def __getJobID(self):
        return self.__job_id
    
    def __setJobID(self, value):
        if not isinstance(value, str):
            raise TypeError("JobID should be a string") 
        self.__job_id = value
        
    def __getJobName(self):
        return self.__job_name
    
    def __setJobName(self,value):
        if not isinstance(value, str):
            raise TypeError("JobName should be a string")
        
    def __getOwnerName(self):
        return self.__owner_name
    
    def __setOwnerName(self,value):
        if not isinstance(value, str):
            raise TypeError("OwnerName should be a string")
        self.__owner_name = value
        
    def __getJobStatus(self):
        return self.__job_status
    
    def __setJobStatus(self, value):
        if not isinstance(value, str):
            raise TypeError("JobStatus should be a string")
        if not value in JobInfo.__STATUS:
            raise ValueError("Invalid status")
        self.__job_status = value
    
    def __getQueueName(self):
        return self.__queue_name
    
    def __setQueueName(self, value):
        if not isinstance(value, str):
            raise TypeError("QueueName should be a string")
        self.__queue_name = value
    
    def __getSubmissionTime(self):
        return self.__submission_time    
    
    def __setSubmissionTime(self,value):
        if not isinstance(value, datetime.datetime):
            raise TypeError("SubmissionTime should be a datetime instance")
        self.__submission_time = value
        
    def __getStartTime(self):
        return self.__start_time    
    
    def __setStartTime(self,value):
        if not (isinstance(value, datetime.datetime) or value == None):
            raise TypeError("StartTime should be a datetime instance")
        self.__start_time = value
        
    def __getEndTime(self):
        return self.__end_time    
    
    def __setEndTime(self,value):
        if not (isinstance(value, datetime.datetime) or value == None):
            raise TypeError("EndTime should be a datetime instance")
        self.__end_time = value
    
    def __getExecutionHosts(self):
        return self.__execution_hosts
    
    def __setExecutionHosts(self, value):
        if not isinstance(value, list):
            raise TypeError("ExecutionHosts should be a list")
        self.__execution_hosts = value
        
    def __getTotalMem(self):
        return self.__total_mem
        
    def __setTotalMem(self,value):
        if (value == None):
            self.__total_mem = None
            return
        if not isinstance(value, int):
            raise TypeError("TotalMem should be an integer")
        if value < 0:
            raise ValueError("TotalMem should be positive number")
        self.__total_mem = value
        
    def __getTotalSwap(self):
        return self.__total_swap
        
    def __setTotalSwap(self,value):
        if (value == None):
            self.__total_swap = None
            return
        if not isinstance(value, int):
            raise TypeError("TotalSwap should be an integer")
        if value < 0:
            raise ValueError("TotalSwap should be positive number")
        self.__total_swap = value
    
    def __getUserTime(self):
        return self.__user_time
        
    def __setUserTime(self,value):
        if (value == None):
            self.__user_time = None
            return
        if not isinstance(value, int):
            raise TypeError("UserTime should be an integer")
        if value < 0:
            raise ValueError("UserTime should be positive number")
        self.__user_time = value
        
    def __getSystemTime(self):
        return self.__system_time
        
    def __setSystemTime(self,value):
        if (value == None):
            self.__system_time = None
            return
        if not isinstance(value, int):
            raise TypeError("UserTime should be an integer")
        if value < 0:
            raise ValueError("UserTime should be positive number")
        self.__system_time = value    
       
    JobID = property(__getJobID, __setJobID, None, \
                     "string, gets/sets job id")
    JobName = property(__getJobName, __setJobName, None, \
                       "string, gets/sets job name")
    OwnerName = property(__getOwnerName, __setOwnerName, None, \
                         "string, gets/sets owner name")
    JobStatus = property(__getJobStatus, __setJobStatus, None, \
                         "string, gets/sets job status, valid values are:\n\
    Pending,\n\
    Running,\n\
    Suspended,\n\
    Done,\n\
    Exited,\n\
    Unknown")
    QueueName = property(__getQueueName, __setQueueName, None, \
                         "string, gets/sets queue name for this job")
    SubmissionTime = property(__getSubmissionTime, __setSubmissionTime, None, \
                              "Datetime, gets/sets job submission time")
    StartTime = property(__getStartTime, __setStartTime, None, \
                         "Datetime, gets/sets job dispatch time (if was dispatched, None otherwise)")
    EndTime = property(__getEndTime, __setEndTime, None, \
                       "Datetime, gets/sets job finish time (if was finished, None otherwise)")
    ExecutionHosts = property(__getExecutionHosts, __setExecutionHosts, None, \
                              "List, gets/sets execution hosts list")
    TotalMem = property(__getTotalMem, __setTotalMem, None, \
                        "int, gets/sets total memory used in MB")
    TotalSwap = property(__getTotalSwap, __setTotalSwap, None, \
                         "int, gets/sets total swap used in MB")
    UserTime = property(__getUserTime, __setUserTime, None, \
                        "int, gets/sets total user time used by job in seconds")
    SystemTime = property(__getSystemTime, __setSystemTime, None, \
                          "int, gets'sets total system time used by job in seconds")
    

__gridMonNSPrefix = "gmon"
__gridMonNS = "http://www.man.poznan.pl/GridMonitoring"
__besFactoryNSPrefix = "bes-factory"
__besFactoryNS = "http://schemas.ggf.org/bes/2006/08/bes-factory"
__jsdlNSPrefix = "jsdl"
__jsdlNS = "http://schemas.ggf.org/jsdl/2005/11/jsdl"

def __getGridMonNSPrefix():
    return __gridMonNSPrefix

def __getGridMonNS():
    return __gridMonNS

def __getBesFactoryNSPrefix():
    return __besFactoryNSPrefix

def __getBesFactoryNS():
    return __besFactoryNS

def __getJsdlNSPrefix():
    return __jsdlNSPrefix

def __getJsdlNS():
    return __jsdlNS

GridMonNSPrefix = property(__getGridMonNSPrefix, doc="Grid Monitoring namespace prefix")
GridMonNS = property(__getGridMonNS, doc="Grid Monitoring namespace")
BesFactoryNSPrefix = property(__getBesFactoryNSPrefix, doc="BESFactory namespace prefix")
BesFactoryNS = property(__getBesFactoryNS, doc="BESFactory namespace")
JsdlNSPrefix = property(__getJsdlNSPrefix, doc="JSDL namespace prefix")
JsdlNS = property(__getJsdlNS, doc="JSDL namespace")
