# $Id$

#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at

#      http://www.apache.org/licenses/LICENSE-2.0

#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import sys
import traceback

_torque_type = "http://schemas.qoscosgrid.org/comp/2011/04/drms/Torque/Torque"
_sge_type = "http://schemas.qoscosgrid.org/comp/2011/04/drms/SunGridEngine/SGE-6.2u5"

# method maps
_capabilities = { }
_jobDataHandlerMap = { }
_queueDataHandlerMap = { }
_hostDataHandlerMap = { }
_reservationDataHandlerMap = { }

#imports
try:
    import pbsimpl
except ImportError:
    _pbs_available = False

try:
    import lsfimpl
except ImportError:
    _lsf_available = False

try:
    import sgeimpl
    _capabilities[_sge_type] = True
    _jobDataHandlerMap[_sge_type] = sgeimpl.getJobData
    _queueDataHandlerMap[_sge_type] = sgeimpl.getQueueData
    _hostDataHandlerMap[_sge_type] = sgeimpl.getHostData
    _reservationDataHandlerMap[_sge_type] = sgeimpl.getReservationData
except ImportError:
    print >> sys.stderr, "Could not load SGE implementation\n", sys.exc_info()
    _capabilities[_sge_type] = False    

try:
    import torqueimpl
    _capabilities[_torque_type] = True
    _jobDataHandlerMap[_torque_type] = torqueimpl.getJobData
    _queueDataHandlerMap[_torque_type] = torqueimpl.getQueueData
    _hostDataHandlerMap[_torque_type] = torqueimpl.getHostData
    _reservationDataHandlerMap[_torque_type] = torqueimpl.getReservationData
except ImportError:
    print >> sys.stderr, "Could not load Torque implementation\n", sys.exc_info()
    _capabilities[_torque_type] = False

def monitoringCapabilities():
    return _capabilities.copy()

def getJobData(system_type, job_ids):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    if not job_ids == None:
        for job in job_ids:
            if not isinstance(job, str):
                raise TypeError, "all jobs ids must be a string"
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _jobDataHandlerMap[system_type](job_ids)
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown" 

def getQueueData(system_type, queue_names):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    if not queue_names == None:
        for queue in queue_names:
            if not isinstance(queue, str):
                raise TypeError, "all queue names must be a string"
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _queueDataHandlerMap[system_type](queue_names)
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"
    
def getHostData(system_type, host_names):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    if not host_names == None:
        for host in host_names:
            if not isinstance(host, str):
                raise TypeError, "all host names must be a string"
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _hostDataHandlerMap[system_type](host_names)
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"
    
def getReservationData(system_type, reservation_ids):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    if not reservation_ids == None:
        for reservation in reservation_ids:
            if not isinstance(reservation, str):
                raise TypeError, "all reservation ids must be a string"
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _reservationDataHandlerMap[system_type](reservation_ids)
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"
