#!/usr/bin/env python

import pbs
import re
from common import HostInfo, QueueInfo, JobInfo

JobsRegex = re.compile(r'(.*)/(.*)')

def getHostData(hostnames=None):
	connection = pbs.Connection()
	hostsInfo = []
	
	if hostnames==None:
		info = connection.get_node_status()
		for host in info:
			hostsInfo.append(decodeHostData(host))
	else:
	
		for hostname in hostnames:
			info = connection.get_node_status([hostname])
			if len(info) < 1:
				continue
			hostsInfo.append(decodeHostData(info[0]))	
	connection.disconnect()
	return hostsInfo
		
def decodeHostData(info):
	current_host = HostInfo()
	current_host.host_name = info[0]
	attributes = info[1]
	current_host.os_name = attributes["resources_available.arch"]
	current_host.cpu_count = attributes["resources_available.ncpus"]
	current_host.total_mem = attributes["resources_available.mem"]
	current_host.used_mem = attributes["resources_assigned.mem"]
	jobs = attributes["jobs"]
	m = JobsRegex.match(jobs)
	while m!=None:
		current_host.jobs.append(m.group(1))
		jobs = m.group(2)
		m = JobsRegex.match(jobs)
	return current_host

def getJobData(jobnames=None):
	connection = pbs.Connection()
	jobsInfo = []
	
	if jobnames==None:
		info = connection.get_job_status()
		for job in info:
			jobsInfo.append(decodeJobData(job))
	else:
	
		for jobname in jobnames:		
			info = connection.get_job_status([jobname])
			if len(info) < 1:
				continue
			jobsInfo.append(decodeJobData(info[0]))	
	connection.disconnect()
	return jobsInfo

def decodeJobData(info):
	current_job = JobInfo()
	current_job.job_id = info[0]
	attributes = info[1]
	current_job.job_name = attributes["Job_Name"]
	current_job.job_status = attributes["job_state"]
	current_job.owner_name = attributes["Job_Owner"]
	current_job.queue_name = attributes["queue"]
	current_job.submission_time = attributes["ctime"]
	current_job.start_time = attributes["etime"]	
	return current_job
	
	
def getQueueData(queuenames=None):
	connection = pbs.Connection()
	queuesInfo = []
	jobsInfo = getJobData()
	if queuenames == None:
		info = connection.get_queue_status()
		for queue in info:
			queuesInfo.append(decodeQueueData(queue, jobsInfo))
	else:	
		for queuename in queuenames:
			info = connection.get_queue_status([queuename])
			if len(info)<1:
				continue
			queuesInfo.append(decodeQueueData(info[0], jobsInfo))		
	return queuesInfo

def decodeQueueData(info, jobsInfo):
	current_queue = QueueInfo()
	current_queue.queue_name = info[0]
	attributes = info[1]
	current_queue.queue_type = attributes["queue_type"]
	current_queue.job_slots_aval = -1
	current_queue.job_slots_total = -1
	if attributes["enabled"] == "True":
		current_queue.queue_state = "enabled"
	else:
		current_queue.queue_state = "disabled"
	for job in jobsInfo:
		if job.queue_name == current_queue.queue_name:
			current_queue.job_ids.append(job.job_id)
	return current_queue
