/* $Id: db.c 20 2009-10-20 17:09:27Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <qcg-core/sm.h>
#include <qcg-core/logger.h>
#include <qcg-core/odbc.h>
#include <qcg-core/xmalloc.h>
#include <qcg-core/modules/modules.h>

#include <db.h>

__SM_ID("$Id: db.c 20 2009-10-20 17:09:27Z mamonski $");

SM_INTERNAL_GENERAL_MODULE = {
        .header = {
                .name = "db"
        }
};

static sm_odbc_handle_t odbc;

sm_status_ret_code_t
smc_db_setup(sm_conf_ctxt_t conf)
{
	const char *db = NULL;
	const char *name = NULL;
	const char *pass = NULL;

	SM_MANDATORY_P(conf);

	SM_DEBUG_ENTER();

	if (sm_conf_node_str(conf, SM_FALSE, "/conf:Database/conf:DSN", &db) != SM_OK)
		return SM_FAULT;

	if (sm_conf_node_str(conf, SM_FALSE, "/conf:Database/conf:User", &name) != SM_OK)
		return SM_FAULT;

	if (sm_conf_node_str(conf, SM_FALSE, "/conf:Database/conf:Password", &pass) == SM_FAULT)
		return SM_FAULT;

	if (!(odbc = sm_odbc_connect(db, name, pass))) {
		const char *p = NULL;
		if (pass)
			p = "...";
		sm_error("Failed to connect to database: name = \"%s\", user = \"%s\", pass = \"%s\"", SM_SAFE_STR(db), SM_SAFE_STR(name), SM_SAFE_STR(p));
		return SM_FAULT;
	}

	return SM_OK;
}

void
smc_db_cleanup(void)
{
	SM_DEBUG_ENTER();

	sm_odbc_disconnect(&odbc);
}

sm_odbc_handle_t
smc_db_get_handle(void)
{
	SM_ASSERT(odbc, "Database module not initialized");

	SM_DEBUG_ENTER();

	return odbc;
}
