/* $Id: identifiers.c 2 2009-10-05 10:09:52Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <qcg-core/sm.h>
#include <qcg-core/logger.h>
#include <qcg-core/transport.h>
#include <qcg-core/uuid.h>
#include <qcg-core/value.h>
#include <qcg-core/xmalloc.h>
#include <qcg-core/modules/modules.h>
#include <qcg-core/modules/atc_transport_module.h>
#include <qcg-core/modules/ecm_gsoap/ecm_gsoap.h>
#include <qcg-core/modules/ecm_gsoap/gsoap/gsoap_buffer.h>

#include <common.h>
#include <identifiers.h>
#include <include/namespaces.h>
#include <stub/qcgcompH.h>
#include <stub/qcgcompStub.h>

__SM_ID("$Id: identifiers.c 2 2009-10-05 10:09:52Z mamonski $");

SM_INTERNAL_GENERAL_MODULE = {
        .header = {
                .name = "identifiers"
        }
};

extern struct Namespace qcgcomp_namespaces[];

char *
smc_generate_id(smc_strdup_function func, void *ctxt)
{
	uuid_t uuid;
	char buf[SM_UUID_SIZE];

	uuid_generate(uuid);
	uuid_unparse(uuid, buf);

	if (func) {
		SM_MANDATORY_P(ctxt);
		return func(ctxt, buf);
	} else
		return sm_strdup(buf);
}

sm_status_ret_code_t
smc_create_activity_epr(sm_transport_t transport, struct wsa5__EndpointReferenceType *ActivityIdentifier, const char *id)
{
	char buf[1024];
	char *serviceurl = NULL;
	sm_atc_transport_module_t atc_module;
	struct soap_dom_element *dom = NULL;

	SM_MANDATORY_P(transport);
	SM_MANDATORY_P(ActivityIdentifier);
	SM_MANDATORY_P(id);

	atc_module = (sm_atc_transport_module_t) transport->atc_transport_modules->data;
	assert(atc_module);

	SM_DEBUG_ENTER_STR(id);

	serviceurl = atc_module->get_service_url((sm_module_t) atc_module, transport);
	if (!serviceurl) {
		sm_error("Couldn't retrieve service URL from transport module");
		goto fault;
	}

	snprintf(buf, sizeof(buf), "<ActivityId xmlns=\"" SMC_NAMESPACE_FACTORY "\">urn:uuid:%s</ActivityId>", id);

	SM_TRY_NOT(dom = transport->transport_malloc(transport, sizeof(struct soap_dom_element)));
	SM_TRY_NOT_OK(sm_gsoap_buffer_get_alloc_soap(buf, dom, xsd__anyType, NULL, qcgcomp_namespaces, ((ecm_gsoap_transport_t) transport)->soap));

	SM_TRY_NOT(ActivityIdentifier->ReferenceParameters = transport->transport_malloc(transport, sizeof(struct wsa5__ReferenceParametersType)));
	soap_default_wsa5__ReferenceParametersType(NULL, ActivityIdentifier->ReferenceParameters);
	ActivityIdentifier->ReferenceParameters->__size = 1;
	ActivityIdentifier->ReferenceParameters->__any = dom;

	SM_TRY_NOT(ActivityIdentifier->Address = transport->transport_strdup(transport, serviceurl));

	SM_FREE(serviceurl);
	return SM_OK;

fault:
	SM_FREE(serviceurl);
	return SM_FAULT;
}

sm_status_ret_code_t
smc_create_reservation_epr(sm_transport_t transport, struct wsa5__EndpointReferenceType *ReservationIdentifier, const char *id)
{
	char buf[1024];
	char *serviceurl = NULL;
	sm_atc_transport_module_t atc_module;
	struct soap_dom_element *dom = NULL;

	SM_MANDATORY_P(transport);
	SM_MANDATORY_P(ReservationIdentifier);
	SM_MANDATORY_P(id);

	atc_module = (sm_atc_transport_module_t) transport->atc_transport_modules->data;
	assert(atc_module);

	SM_DEBUG_ENTER_STR(id);

	serviceurl = atc_module->get_service_url((sm_module_t) atc_module, transport);
	if (!serviceurl) {
		sm_error("Couldn't retrieve service URL from transport module");
		goto fault;
	}

	snprintf(buf, sizeof(buf), "<ReservationId xmlns=\"" SMC_NAMESPACE_ARES "\">urn:uuid:%s</ReservationId>", id);

	SM_TRY_NOT(dom = transport->transport_malloc(transport, sizeof(struct soap_dom_element)));
	SM_TRY_NOT_OK(sm_gsoap_buffer_get_alloc_soap(buf, dom, xsd__anyType, NULL, qcgcomp_namespaces, ((ecm_gsoap_transport_t) transport)->soap));

	SM_TRY_NOT(ReservationIdentifier->ReferenceParameters = transport->transport_malloc(transport, sizeof(struct wsa5__ReferenceParametersType)));
	soap_default_wsa5__ReferenceParametersType(NULL, ReservationIdentifier->ReferenceParameters);
	ReservationIdentifier->ReferenceParameters->__size = 1;
	ReservationIdentifier->ReferenceParameters->__any = dom;

	SM_TRY_NOT(ReservationIdentifier->Address = transport->transport_strdup(transport, serviceurl));

	SM_FREE(serviceurl);
	return SM_OK;

fault:
	SM_FREE(serviceurl);
	return SM_FAULT;
}

sm_status_ret_code_t
smc_parse_activity_epr(struct wsa5__EndpointReferenceType *ActivityIdentifier, const char **id)
{
	int i;

	SM_MANDATORY_P(ActivityIdentifier);
	SM_MANDATORY_P(id);

	SM_DEBUG_ENTER();

	*id = NULL;

	if (ActivityIdentifier->ReferenceParameters) {

		sm_debug("Reference parameters count = %d", ActivityIdentifier->ReferenceParameters->__size);

		for (i = 0; i < ActivityIdentifier->ReferenceParameters->__size; i++) {
			struct soap_dom_element *dom = ActivityIdentifier->ReferenceParameters->__any + i;

			sm_debug("dom->type = %d", dom->type);

			if (dom->type == SOAP_TYPE__qcg_comp_factory__ActivityId) {
				const char *value = (const char *) dom->node;
				size_t ignore = 9; /* == strlen("urn:uuid:") */

				if (strlen(value) != 45) { /* == strlen("urn:uuid:550e8400-e29b-41d4-a716-446655440000") */
					sm_error("Illegal length of the activity identifier: %lu", (unsigned long)strlen(value));
					return SM_FAULT;
				}

				if (strncmp(value, "urn:uuid:", ignore))
					return SM_FAULT;

				sm_debug("Parsed activity EPR: %s", value + ignore);
				*id = value + ignore;
				return SM_OK;
			}
		}
	}

	return SM_NEG;
}

sm_status_ret_code_t
smc_parse_reservation_epr(struct wsa5__EndpointReferenceType *ReservationIdentifier, const char **id)
{
	int i;

	SM_MANDATORY_P(ReservationIdentifier);
	SM_MANDATORY_P(id);

	SM_DEBUG_ENTER();

	*id = NULL;

	if (ReservationIdentifier->ReferenceParameters) {

		sm_debug("Reference parameters count = %d", ReservationIdentifier->ReferenceParameters->__size);

		for (i = 0; i < ReservationIdentifier->ReferenceParameters->__size; i++) {
			struct soap_dom_element *dom = ReservationIdentifier->ReferenceParameters->__any + i;

			sm_debug("dom->type = %d", dom->type);

			if (dom->type == SOAP_TYPE__qcg_comp_ares__ReservationId) {
				const char *value = (const char *) dom->node;
				size_t ignore = 9; /* == strlen("urn:uuid:") */

				if (strlen(value) != 45) { /* == strlen("urn:uuid:550e8400-e29b-41d4-a716-446655440000") */
					sm_error("Illegal length of the reservation identifier: %lu", (unsigned long)strlen(value));
					return SM_FAULT;
				}

				if (strncmp(value, "urn:uuid:", ignore))
					return SM_FAULT;

				sm_debug("Parsed reservation EPR: %s", value + ignore);
				*id = value + ignore;
				return SM_OK;
			}
		}
	}

	return SM_NEG;
}
