/* $Id: job.h 217 2010-08-06 07:24:42Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef __SMC_JOB_H
#define __SMC_JOB_H

#include <stdbool.h>
#include <time.h>

#include <qcg-core/sm.h>
#include <qcg-core/list.h>
#include <qcg-core/odbc.h>
#include <qcg-core/value.h>

#include <stub/qcgcompStub.h>

typedef enum {
	SMC_JOB_OK = SM_OK,
	SMC_JOB_INTERNAL_FAULT,
	SMC_JOB_DATABASE_FAULT,
	SMC_JOB_NOT_FOUND,
	SMC_JOB_INVALID_ARGUMENT
} smc_job_ret_code_t;

typedef enum {
	SMC_UNSPECIFIED_STATE = 0,
	SMC_STAGE_IN = 1,
	SMC_HELD = 2,
	SMC_QUEUED = 3,
	SMC_EXECUTING = 4,
	SMC_SUSPENDED = 5,
	SMC_STAGE_OUT = 6,
	SMC_FINISHED = 7,
	SMC_FAILED = 8,
	SMC_CANCELLED = 9
} smc_job_state_t;

#define SMC_JOB_STATE_TERMINAL(state)	((state) == SMC_FINISHED || (state) == SMC_FAILED || (state) == SMC_CANCELLED)

struct smc_job_s;

typedef struct smc_job_res_info_s {
	const char *drms_id;
	unsigned int duration;
	SM_BOOL cancelled;
	SM_BOOL expired;
} *smc_job_res_info_t;

typedef const char *(*smc_job_string_getter_function)(struct smc_job_s *this);
typedef _jsdl__JobDefinition *(*smc_job_jsdl_parsed_getter_function)(struct smc_job_s *this);
typedef smc_job_state_t (*smc_job_state_getter_function)(struct smc_job_s *this);
typedef struct tm (*smc_job_time_getter_function)(struct smc_job_s *this);
typedef unsigned char (*smc_job_byte_getter_function)(struct smc_job_s *this);
typedef int (*smc_job_int_getter_function)(struct smc_job_s *this);
typedef bool (*smc_job_bool_getter_function)(struct smc_job_s *this);
typedef double (*smc_job_double_getter_function)(struct smc_job_s *this);
typedef smc_job_res_info_t (*smc_job_reservation_info_getter_function)(struct smc_job_s *this);
typedef sm_value_t *(*smc_job_value_getter_function)(struct smc_job_s *this);

typedef const char *(*smc_job_string_setter_function)(struct smc_job_s *this, const char *value);
typedef const char *(*smc_job_jsdl_parsed_setter_function)(struct smc_job_s *this, _jsdl__JobDefinition *value);
typedef smc_job_state_t (*smc_job_state_setter_function)(struct smc_job_s *this, smc_job_state_t value);
typedef struct tm (*smc_job_time_setter_function)(struct smc_job_s *this, struct tm value);
typedef unsigned char (*smc_job_byte_setter_function)(struct smc_job_s *this, unsigned char value);
typedef int (*smc_job_int_setter_function)(struct smc_job_s *this, int value);
typedef bool (*smc_job_bool_setter_function)(struct smc_job_s *this, bool value);
typedef double (*smc_job_double_setter_function)(struct smc_job_s *this, double value);
typedef sm_value_t *(*smc_job_value_setter_function)(struct smc_job_s *this, sm_value_t *value);

typedef sm_ret_code_t (*smc_job_save_function)(struct smc_job_s *this);

typedef struct smc_job_s {
	smc_job_string_getter_function get_id;
	smc_job_string_getter_function get_drms_id;
	smc_job_string_getter_function get_reservation_id;
	smc_job_reservation_info_getter_function get_reservation_info;
	smc_job_string_getter_function get_submitted_jsdl;
	smc_job_string_getter_function get_jsdl;
	smc_job_jsdl_parsed_getter_function get_jsdl_parsed;
	smc_job_string_getter_function get_subject;
	smc_job_string_getter_function get_mapped_user;
	smc_job_state_getter_function get_state;
	smc_job_string_getter_function get_staging_fault;
	smc_job_time_getter_function get_submission_time;
	smc_job_time_getter_function get_start_time;
	smc_job_time_getter_function get_end_time;
	smc_job_byte_getter_function get_exit_status;
	smc_job_bool_getter_function has_exit_status;
	smc_job_bool_getter_function get_core_dumped;
	smc_job_bool_getter_function has_core_dumped;
	smc_job_string_getter_function get_posix_signal_name;
	smc_job_double_getter_function get_cpu_usage;
	smc_job_double_getter_function get_mem_usage;
	smc_job_double_getter_function get_vmem_usage;
	smc_job_int_getter_function get_walltime;
	smc_job_string_getter_function get_project;
	smc_job_string_getter_function get_queue;
	smc_job_string_getter_function get_nodes;
	smc_job_value_getter_function get_atc_token;

	smc_job_string_setter_function set_id;
	smc_job_string_setter_function set_drms_id;
	smc_job_string_setter_function set_submitted_jsdl;
	smc_job_string_setter_function set_jsdl;
	smc_job_jsdl_parsed_setter_function set_jsdl_parsed;
	smc_job_string_setter_function set_subject;
	smc_job_string_setter_function set_mapped_user;
	smc_job_state_setter_function set_state;
	smc_job_string_setter_function set_staging_fault;
	smc_job_time_setter_function set_start_time;
	smc_job_byte_setter_function set_exit_status;
	smc_job_bool_setter_function set_core_dumped;
	smc_job_string_setter_function set_posix_signal_name;
	smc_job_double_setter_function set_cpu_usage;
	smc_job_double_setter_function set_mem_usage;
	smc_job_double_setter_function set_vmem_usage;
	smc_job_int_setter_function set_walltime;
	smc_job_string_setter_function set_project;
	smc_job_string_setter_function set_queue;
	smc_job_string_setter_function set_nodes;
	smc_job_value_setter_function set_atc_token;

	smc_job_bool_getter_function is_readonly;
	smc_job_save_function save;
} *smc_job_t;

sm_status_ret_code_t smc_job_setup(sm_odbc_handle_t odbc_handle, bool session_process);
void smc_job_cleanup(void);

smc_job_t smc_job_create(void);
void smc_job_free(smc_job_t *job);

sm_ret_code_t smc_job_get(const char *id, sm_list_t *result);
sm_ret_code_t smc_job_get_by_drms_id(const char *drms_id, smc_job_t *result);

sm_ret_code_t smc_job_get_readonly(const char *id, sm_list_t *result);
sm_ret_code_t smc_job_get_by_drms_id_readonly(const char *drms_id, smc_job_t *result);

sm_ret_code_t smc_job_get_active_readonly(sm_list_t *result);
int smc_job_count(void);

const char *smc_job_state_str(smc_job_state_t state);
smc_job_state_t smc_job_state_parse(const char *state);

#endif /* __SMC_JOB_H */
