#ifndef QCG_COMP_SOCKS_SERVER_H
#define QCG_COMP_SOCKS_SERVER_H

#include <time.h>
#include <pthread.h>
#include <qcg-core/sm.h>
#include <netinet/in.h>

typedef enum { smc_socks_atn_none, smc_socks_atn_per_session, smc_socks_atn_per_connection } smc_socks_authentication_t;

typedef struct smc_socks_token_s {
	char *token;
	time_t registered;
} *smc_socks_token_t;

typedef struct smc_socks_tokens_s {
	smc_socks_token_t *data;
	size_t size;
	pthread_mutex_t lock;

	SM_BOOL (*add)(struct smc_socks_tokens_s *tokens, const char *token);
	SM_BOOL (*check)(struct smc_socks_tokens_s *tokens, const char *token);
} *smc_socks_tokens_t;

typedef struct smc_socks_allowed_hosts_s {
	in_addr_t address;
	in_addr_t mask;
} *smc_socks_allowed_hosts_t;

typedef struct smc_socks_allowed_ports_s {
	int min_port;
	int max_port;
} *smc_socks_allowed_ports_t;

typedef struct smc_socks_config_s {
	char* socks_port;
	smc_socks_authentication_t authentication;

	int allowed_src_hosts_count;
	int allowed_dest_hosts_count;
	int allowed_dest_ports_count;

	smc_socks_allowed_hosts_t allowed_src_hosts;
	smc_socks_allowed_hosts_t allowed_dst_hosts;
	smc_socks_allowed_ports_t allowed_ports;

	smc_socks_tokens_t tokens;

} *smc_socks_config_t;

void* socks_main(smc_socks_config_t config);

#endif // QCG_COMP_SOCKS_SERVER_H
