/* $Id: qcg_coordinator_client.h 2 2009-10-05 10:09:52Z mamonski $ */

/*
 * Copyright 2006-2009 QCG Project Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __ECM_QCG_COORDINATOR_CLIENT_H
#define __ECM_QCG_COORDINATOR_CLIENT_H

#include <qcg-core/value.h>
#include <qcg-core/transport.h>

typedef struct smc_coordinator_process_entry_s {
	char* key_name;
	char* master_url;
	sm_value_t* value;
} *smc_coordinator_process_entry_t;

typedef int (*smc_coordinator_GetProcessEntry_call_function)(sm_client_transport_t, int*, smc_coordinator_process_entry_t*, int, smc_coordinator_process_entry_t**, int*);
typedef int (*smc_coordinator_PutProcessEntry_call_function)(sm_client_transport_t, smc_coordinator_process_entry_t*, int);
typedef int (*smc_coordinator_Elect_call_function)(sm_client_transport_t, char*, SM_BOOL*);

typedef struct smc_coordinator_client_api_s{
	smc_coordinator_GetProcessEntry_call_function get;
	smc_coordinator_PutProcessEntry_call_function put;
	smc_coordinator_Elect_call_function elect;
} *smc_coordinator_client_api_t;

static smc_coordinator_process_entry_t
smc_coordinator_process_entry_create(char* key_name,
		char* master_url,
		sm_value_t* value)
{
	smc_coordinator_process_entry_t result;

	result = sm_malloc(sizeof(*result));

	if (result == NULL) {
		return NULL;
	}

	result->key_name = sm_strdup(key_name);
	result->master_url = master_url == NULL ? NULL : sm_strdup(master_url);
	result->value = value;

	return result;
}

static void
smc_coordinator_process_entry_destroy(smc_coordinator_process_entry_t* pe,
		SM_BOOL free_value)
{
	sm_free((*pe)->key_name);
	sm_free((*pe)->master_url);

	if (free_value) {
		sm_value_free(&(*pe)->value);
	}

	sm_free(*pe);
	*pe = NULL;
}

#endif

