#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <rsync/interface.h>
#include <qcg-core/modules/general_module.h>
#include <qcg-core/logger.h>

SM_INTERNAL_GENERAL_MODULE = {
        .header = {
                .name = "rsync-icomm"
        }
};


int
icomm_to_signature(rsync_transport_t transport, sm_value_t *result,Signature *sig)
{
	int *ptr = NULL;

	SM_DEBUG_ENTER_INT(result[2]->vint32);

	sig->BlockLength = result[0]->vint32;
	sig->StrongSumLength = result[1]->vint32;
	sig->__sizeSignatures = result[2]->vint32;
	sig->Length = result[3]->vint32;
	sig->Remainder = result[4]->vint32;
	sig->Signatures = transport->transport_malloc(transport, sig->__sizeSignatures * sizeof(struct qcg_comp_rsync__BlockSignatureType));

	if(sig->__sizeSignatures > 0 && !sig->Signatures)
		return SM_DEBUG_LEAVE_INT(0);

	ptr = result[5]->vint32_array;

	for (int i = 0; i < sig->__sizeSignatures; ++i)
		sig->Signatures[i].Weak = ptr[i];

	for (int i = 0; i < sig->__sizeSignatures; ++i) {
		sig->Signatures[i].Strong = transport->transport_strdup(transport, result[6]->vstrings[i]);

		if(!sig->Signatures[i].Strong)
			return SM_DEBUG_LEAVE_INT(0);
	}

	return SM_DEBUG_LEAVE_INT(1);
}

int
signature_to_icomm(rsync_transport_t transport, const Signature *sig, sm_value_t **response)
{
	int i;
	int *weak = NULL;
	char **strong = NULL;
	size_t size_signatures = (size_t)sig->__sizeSignatures;

	SM_DEBUG_ENTER();

	sm_debug("sig->__sizeSignatures = %i", sig->__sizeSignatures);

	if(sig->__sizeSignatures > 0) {
		SM_TRY_NOT(weak = sm_calloc(sig->__sizeSignatures, sizeof(int)));
		SM_TRY_NOT(strong = sm_calloc(sig->__sizeSignatures, sizeof(char*)));
	}

	for (i = 0; i < sig->__sizeSignatures; ++i) {
		weak[i]=sig->Signatures[i].Weak;
		SM_TRY_NOT(strong[i] = sm_strdup(sig->Signatures[i].Strong));
	}

	if(sm_value_append(response, 7,
				SM_VALUE_INT32, sig->BlockLength,
				SM_VALUE_INT32, sig->StrongSumLength,
				SM_VALUE_INT32, sig->__sizeSignatures,
				SM_VALUE_INT32, sig->Length,
				SM_VALUE_INT32, sig->Remainder,
				SM_VALUE_INT32_ARRAY|SM_VALUE_DONT_COPY_FLAG, size_signatures, weak,
				SM_VALUE_STRINGS|SM_VALUE_DONT_COPY_FLAG, size_signatures , strong
				))
		return SM_DEBUG_LEAVE_INT(1);
	else {
		return SM_DEBUG_LEAVE_INT(0);
	}

fault:

	if(strong) {

		for (i = 0; i < sig->__sizeSignatures; ++i) {
			sm_free(strong[i]);
		}

		sm_free(strong);
	}

	sm_free(weak);

	return SM_DEBUG_LEAVE_INT(0);
}
